# -*- coding: utf-8 -*-
from anytree import LightNodeMixin, LoopError, PostOrderIter, PreOrderIter, TreeError
from anytree.node.util import _repr

from .helper import assert_raises


class LightNode(LightNodeMixin):

    __slots__ = ["name"]

    def __init__(self, name, parent=None, children=None):
        self.name = name
        self.parent = parent
        if children:
            self.children = children

    def __repr__(self):
        path = self.separator.join([""] + [str(node.name) for node in self.path])
        return "%s(%r)" % (self.__class__.__name__, path)


def test_parent_child():
    """A tree parent and child attributes."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1a = LightNode("sub1A", parent=s1)
    s1b = LightNode("sub1B", parent=s1)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.parent is None
    assert root.children == tuple([s0, s1])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent == root
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple([s1ca])
    assert s1ca.parent == s1c
    assert s1ca.children == tuple()

    # change parent
    s1ca.parent = s0

    assert root.parent is None
    assert root.children == tuple([s0, s1])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a, s1ca])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent == root
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple()
    assert s1ca.parent == s0
    assert s1ca.children == tuple()

    # break tree into two
    s1.parent = None

    assert root.parent is None
    assert root.children == tuple([s0])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a, s1ca])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent is None
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple()
    assert s1ca.parent == s0
    assert s1ca.children == tuple()

    # set to the same
    s1b.parent = s1

    assert root.parent is None
    assert root.children == tuple([s0])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a, s1ca])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent is None
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple()
    assert s1ca.parent == s0
    assert s1ca.children == tuple()


def test_detach_children():

    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1a = LightNode("sub1A", parent=s1)
    s1b = LightNode("sub1B", parent=s1)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.descendants == (s0, s0b, s0a, s1, s1a, s1b, s1c, s1ca)
    del s0.children
    assert root.descendants == (s0, s1, s1a, s1b, s1c, s1ca)
    del s1.children
    assert root.descendants == (s0, s1)


def test_children_setter():

    root = LightNode("root")
    s0 = LightNode("sub0")
    s1 = LightNode("sub0A")
    s0a = LightNode("sub0B")

    root.children = [s0, s1]
    s0.children = [s0a]
    assert root.descendants == (s0, s0a, s1)

    with assert_raises(LoopError, "Cannot set parent. LightNode('/root/sub0') cannot be parent of itself."):
        s0.children = [s0]

    # test whether tree is unchanged after LoopError
    assert root.descendants == (s0, s0a, s1)

    with assert_raises(
        LoopError, "Cannot set parent. LightNode('/root/sub0') is parent of LightNode('/root/sub0/sub0B')."
    ):
        s0a.children = [s0]

    # test whether tree is unchanged after LoopError
    assert root.descendants == (s0, s0a, s1)

    root.children = [s0, s1]
    s0.children = [s0a]
    s0a.children = [s1]
    assert root.descendants == (s0, s0a, s1)


def test_children_setter_large():

    root = LightNode("root")
    s0 = LightNode("sub0")
    s0b = LightNode("sub0B")
    s0a = LightNode("sub0A")
    s1 = LightNode("sub1")
    s1a = LightNode("sub1A")
    s1b = LightNode("sub1B")
    s1c = LightNode("sub1C")
    s1ca = LightNode("sub1Ca")

    root.children = [s0, s1]
    assert root.descendants == (s0, s1)
    s0.children = [s0a, s0b]
    assert root.descendants == (s0, s0a, s0b, s1)
    s1.children = [s1a, s1b, s1c]
    assert root.descendants == (s0, s0a, s0b, s1, s1a, s1b, s1c)
    with assert_raises(TypeError, "'LightNode' object is not iterable"):
        s1.children = s1ca
    assert root.descendants == (s0, s0a, s0b, s1, s1a, s1b, s1c)


def test_node_children_multiple():

    root = LightNode("root")
    sub = LightNode("sub")
    with assert_raises(TreeError, "Cannot add node LightNode('/sub') multiple times as child."):
        root.children = [sub, sub]


def test_recursion_detection():
    """Recursion detection."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)

    # try recursion
    assert root.parent is None
    try:
        root.parent = root
    except LoopError as exc:
        assert str(exc) == "Cannot set parent. LightNode('/root') cannot be parent of itself."
        assert root.parent is None
    else:
        assert False

    assert root.parent is None
    try:
        root.parent = s0a
    except LoopError as exc:
        assert str(exc) == ("Cannot set parent. LightNode('/root') is parent of LightNode('/root/sub0/sub0A').")
        assert root.parent is None
    else:
        assert False

    assert s0.parent is root
    try:
        s0.parent = s0a
    except LoopError as exc:
        assert str(exc) == ("Cannot set parent. LightNode('/root/sub0') is parent of LightNode('/root/sub0/sub0A').")
        assert s0.parent is root
    else:
        assert False


def test_ancestors():
    """Node.ancestors."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.ancestors == tuple()
    assert s0.ancestors == tuple([root])
    assert s0b.ancestors == tuple([root, s0])
    assert s0a.ancestors == tuple([root, s0])
    assert s1ca.ancestors == tuple([root, s1, s1c])


def test_node_children_init():
    """Node With Children Attribute."""
    root = LightNode("root", children=[LightNode("a", children=[LightNode("aa")]), LightNode("b")])
    assert repr(root.descendants) == "(LightNode('/root/a'), LightNode('/root/a/aa'), LightNode('/root/b'))"


def test_descendants():
    """Node.descendants."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.descendants == tuple([s0, s0b, s0a, s1, s1c, s1ca])
    assert s1.descendants == tuple([s1c, s1ca])
    assert s1c.descendants == tuple([s1ca])
    assert s1ca.descendants == tuple()


def test_root():
    """Node.root."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.root == root
    assert s0.root == root
    assert s0b.root == root
    assert s0a.root == root
    assert s1.root == root
    assert s1c.root == root
    assert s1ca.root == root


def test_siblings():
    """Node.siblings."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.siblings == tuple()
    assert s0.siblings == tuple([s1])
    assert s0b.siblings == tuple([s0a])
    assert s0a.siblings == tuple([s0b])
    assert s1.siblings == tuple([s0])
    assert s1c.siblings == tuple()
    assert s1ca.siblings == tuple()


def test_is_leaf():
    """Node.is_leaf."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.is_leaf is False
    assert s0.is_leaf is False
    assert s0b.is_leaf is True
    assert s0a.is_leaf is True
    assert s1.is_leaf is False
    assert s1c.is_leaf is False
    assert s1ca.is_leaf is True


def test_leaves():
    """Node.leaves."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.leaves == tuple([s0b, s0a, s1ca])
    assert s0.leaves == tuple([s0b, s0a])
    assert s0b.leaves == tuple([s0b])
    assert s0a.leaves == tuple([s0a])
    assert s1.leaves == tuple([s1ca])
    assert s1c.leaves == tuple([s1ca])
    assert s1ca.leaves == tuple([s1ca])


def test_is_root():
    """Node.is_root."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.is_root is True
    assert s0.is_root is False
    assert s0b.is_root is False
    assert s0a.is_root is False
    assert s1.is_root is False
    assert s1c.is_root is False
    assert s1ca.is_root is False


def test_height():
    """Node.height."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.height == 3
    assert s0.height == 1
    assert s0b.height == 0
    assert s0a.height == 0
    assert s1.height == 2
    assert s1c.height == 1
    assert s1ca.height == 0


def test_size():
    """Node.size."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.size == 7
    assert s0.size == 3
    assert s0b.size == 1
    assert s0a.size == 1
    assert s1.size == 3
    assert s1c.size == 2
    assert s1ca.size == 1


def test_size():
    """Node.size."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.size == 7
    assert s0.size == 3
    assert s0b.size == 1
    assert s0a.size == 1
    assert s1.size == 3
    assert s1c.size == 2
    assert s1ca.size == 1


def test_depth():
    """Node.depth."""
    root = LightNode("root")
    s0 = LightNode("sub0", parent=root)
    s0b = LightNode("sub0B", parent=s0)
    s0a = LightNode("sub0A", parent=s0)
    s1 = LightNode("sub1", parent=root)
    s1c = LightNode("sub1C", parent=s1)
    s1ca = LightNode("sub1Ca", parent=s1c)

    assert root.depth == 0
    assert s0.depth == 1
    assert s0b.depth == 2
    assert s0a.depth == 2
    assert s1.depth == 1
    assert s1c.depth == 2
    assert s1ca.depth == 3


def test_parent():
    """Parent attribute."""
    foo = LightNodeMixin()
    assert foo.parent is None


def test_pre_order_iter():
    """Pre-Order Iterator."""
    f = LightNode("f")
    b = LightNode("b", parent=f)
    a = LightNode("a", parent=b)
    d = LightNode("d", parent=b)
    c = LightNode("c", parent=d)
    e = LightNode("e", parent=d)
    g = LightNode("g", parent=f)
    i = LightNode("i", parent=g)
    h = LightNode("h", parent=i)

    assert [node.name for node in PreOrderIter(f)] == ["f", "b", "a", "d", "c", "e", "g", "i", "h"]


def test_post_order_iter():
    """Post-Order Iterator."""
    f = LightNode("f")
    b = LightNode("b", parent=f)
    a = LightNode("a", parent=b)
    d = LightNode("d", parent=b)
    c = LightNode("c", parent=d)
    e = LightNode("e", parent=d)
    g = LightNode("g", parent=f)
    i = LightNode("i", parent=g)
    h = LightNode("h", parent=i)

    assert [node.name for node in PostOrderIter(f)] == ["a", "c", "e", "d", "b", "h", "i", "g", "f"]


def test_hookups():
    """Hookup attributes #29."""

    class MyLightNode(LightNode):
        def _pre_attach(self, parent):
            assert str(self.parent) == "None"
            assert self.children == tuple()
            assert str(self.path) == "(MyLightNode('/B'),)"

        def _post_attach(self, parent):
            assert str(self.parent) == "MyLightNode('/A')"
            assert self.children == tuple()
            assert str(self.path) == "(MyLightNode('/A'), MyLightNode('/A/B'))"

        def _pre_detach(self, parent):
            assert str(self.parent) == "MyLightNode('/A')"
            assert self.children == tuple()
            assert str(self.path) == "(MyLightNode('/A'), MyLightNode('/A/B'))"

        def _post_detach(self, parent):
            assert str(self.parent) == "None"
            assert self.children == tuple()
            assert str(self.path) == "(MyLightNode('/B'),)"

    node_a = MyLightNode("A")
    node_b = MyLightNode("B", node_a)  # attach B on A
    node_b.parent = None  # detach B from A



"""Helper Methods for testing."""
from contextlib import contextmanager

import six


def eq_(one, other):
    assert one == other, "{one} != {other}".format(one=one, other=other)


# hack own assert_raises, because py26 has a different impelmentation
@contextmanager
def assert_raises(exccls, msg):
    """Check exception of class `exccls` to be raised with message `msg`."""
    try:
        yield
        assert False, "%r not raised" % exccls
    except Exception as exc:
        assert isinstance(exc, exccls), "%r is not a %r" % (exc, exccls)
        eq_(str(exc), msg)


def with_setup(setup=None, teardown=None):
    def decorate(func, setup=setup, teardown=teardown):
        if setup:
            if hasattr(func, "setup"):
                _old_s = func.setup

                def _s():
                    setup()
                    _old_s()

                func.setup = _s
            else:
                func.setup = setup
        if teardown:
            if hasattr(func, "teardown"):
                _old_t = func.teardown

                def _t():
                    _old_t()
                    teardown()

                func.teardown = _t
            else:
                func.teardown = teardown
        return func

    return decorate



# -*- coding: utf-8 -*-
from anytree import AnyNode, LoopError, Node, NodeMixin, PostOrderIter, PreOrderIter, RenderTree, TreeError

from .helper import assert_raises, eq_


class TNode(Node):

    TRACKING = []

    def _pre_detach(self, parent):
        """Method call before detaching from `parent`."""
        self.TRACKING.append("_pre_detach(%r, %r)" % (self.name, parent.name))

    def _post_detach(self, parent):
        """Method call after detaching from `parent`."""
        self.TRACKING.append("_post_detach(%r, %r)" % (self.name, parent.name))

    def _pre_attach(self, parent):
        """Method call before attaching to `parent`."""
        self.TRACKING.append("_pre_attach(%r, %r)" % (self.name, parent.name))

    def _post_attach(self, parent):
        """Method call after attaching to `parent`."""
        self.TRACKING.append("_post_attach(%r, %r)" % (self.name, parent.name))

    def _pre_detach_children(self, children):
        """Method call before detaching `children`."""
        self.TRACKING.append("_pre_detach_children(%r, %r)" % (self.name, tuple(child.name for child in children)))

    def _post_detach_children(self, children):
        """Method call after detaching `children`."""
        self.TRACKING.append("_post_detach_children(%r, %r)" % (self.name, tuple(child.name for child in children)))

    def _pre_attach_children(self, children):
        """Method call before attaching `children`."""
        self.TRACKING.append("_pre_attach_children(%r, %r)" % (self.name, tuple(child.name for child in children)))

    def _post_attach_children(self, children):
        """Method call after attaching `children`."""
        self.TRACKING.append("_post_attach_children(%r, %r)" % (self.name, tuple(child.name for child in children)))


def test_parent_child():
    """A tree parent and child attributes."""
    root = TNode("root")
    s0 = TNode("sub0", parent=root)
    s0b = TNode("sub0B", parent=s0)
    s0a = TNode("sub0A", parent=s0)
    s1 = TNode("sub1", parent=root)
    s1a = TNode("sub1A", parent=s1)
    s1b = TNode("sub1B", parent=s1)
    s1c = TNode("sub1C", parent=s1)
    s1ca = TNode("sub1Ca", parent=s1c)

    assert TNode.TRACKING == [
        "_pre_attach('sub0', 'root')",
        "_post_attach('sub0', 'root')",
        "_pre_attach('sub0B', 'sub0')",
        "_post_attach('sub0B', 'sub0')",
        "_pre_attach('sub0A', 'sub0')",
        "_post_attach('sub0A', 'sub0')",
        "_pre_attach('sub1', 'root')",
        "_post_attach('sub1', 'root')",
        "_pre_attach('sub1A', 'sub1')",
        "_post_attach('sub1A', 'sub1')",
        "_pre_attach('sub1B', 'sub1')",
        "_post_attach('sub1B', 'sub1')",
        "_pre_attach('sub1C', 'sub1')",
        "_post_attach('sub1C', 'sub1')",
        "_pre_attach('sub1Ca', 'sub1C')",
        "_post_attach('sub1Ca', 'sub1C')",
    ]
    TNode.TRACKING.clear()

    # change parent
    s1ca.parent = s0

    # break tree into two
    s1.parent = None

    # set to the same
    s1b.parent = s1

    assert TNode.TRACKING == [
        "_pre_detach('sub1Ca', 'sub1C')",
        "_post_detach('sub1Ca', 'sub1C')",
        "_pre_attach('sub1Ca', 'sub0')",
        "_post_attach('sub1Ca', 'sub0')",
        "_pre_detach('sub1', 'root')",
        "_post_detach('sub1', 'root')",
    ]
    TNode.TRACKING.clear()


def test_detach_children():

    root = TNode("root")
    s0 = TNode("sub0", parent=root)
    s0b = TNode("sub0B", parent=s0)
    s0a = TNode("sub0A", parent=s0)
    s1 = TNode("sub1", parent=root)
    s1a = TNode("sub1A", parent=s1)
    s1b = TNode("sub1B", parent=s1)
    s1c = TNode("sub1C", parent=s1)
    s1ca = TNode("sub1Ca", parent=s1c)

    assert TNode.TRACKING == [
        "_pre_attach('sub0', 'root')",
        "_post_attach('sub0', 'root')",
        "_pre_attach('sub0B', 'sub0')",
        "_post_attach('sub0B', 'sub0')",
        "_pre_attach('sub0A', 'sub0')",
        "_post_attach('sub0A', 'sub0')",
        "_pre_attach('sub1', 'root')",
        "_post_attach('sub1', 'root')",
        "_pre_attach('sub1A', 'sub1')",
        "_post_attach('sub1A', 'sub1')",
        "_pre_attach('sub1B', 'sub1')",
        "_post_attach('sub1B', 'sub1')",
        "_pre_attach('sub1C', 'sub1')",
        "_post_attach('sub1C', 'sub1')",
        "_pre_attach('sub1Ca', 'sub1C')",
        "_post_attach('sub1Ca', 'sub1C')",
    ]
    TNode.TRACKING.clear()

    del s0.children

    assert TNode.TRACKING == [
        "_pre_detach_children('sub0', ('sub0B', 'sub0A'))",
        "_pre_detach('sub0B', 'sub0')",
        "_post_detach('sub0B', 'sub0')",
        "_pre_detach('sub0A', 'sub0')",
        "_post_detach('sub0A', 'sub0')",
        "_post_detach_children('sub0', ('sub0B', 'sub0A'))",
    ]
    TNode.TRACKING.clear()

    del s1.children

    assert TNode.TRACKING == [
        "_pre_detach_children('sub1', ('sub1A', 'sub1B', 'sub1C'))",
        "_pre_detach('sub1A', 'sub1')",
        "_post_detach('sub1A', 'sub1')",
        "_pre_detach('sub1B', 'sub1')",
        "_post_detach('sub1B', 'sub1')",
        "_pre_detach('sub1C', 'sub1')",
        "_post_detach('sub1C', 'sub1')",
        "_post_detach_children('sub1', ('sub1A', 'sub1B', 'sub1C'))",
    ]
    TNode.TRACKING.clear()


def test_children_setter():

    root = TNode("root")
    s0 = TNode("sub0")
    s1 = TNode("sub0A")
    s0a = TNode("sub0B")

    assert TNode.TRACKING == []
    TNode.TRACKING.clear()

    root.children = [s0, s1]
    s0.children = [s0a]

    assert TNode.TRACKING == [
        "_pre_detach_children('root', ())",
        "_post_detach_children('root', ())",
        "_pre_attach_children('root', ('sub0', 'sub0A'))",
        "_pre_attach('sub0', 'root')",
        "_post_attach('sub0', 'root')",
        "_pre_attach('sub0A', 'root')",
        "_post_attach('sub0A', 'root')",
        "_post_attach_children('root', ('sub0', 'sub0A'))",
        "_pre_detach_children('sub0', ())",
        "_post_detach_children('sub0', ())",
        "_pre_attach_children('sub0', ('sub0B',))",
        "_pre_attach('sub0B', 'sub0')",
        "_post_attach('sub0B', 'sub0')",
        "_post_attach_children('sub0', ('sub0B',))",
    ]
    TNode.TRACKING.clear()

    with assert_raises(LoopError, "Cannot set parent. TNode('/root/sub0') cannot be parent of itself."):
        s0.children = [s0]

    # test whether tree is unchanged after LoopError
    assert TNode.TRACKING == [
        "_pre_detach_children('sub0', ('sub0B',))",
        "_pre_detach('sub0B', 'sub0')",
        "_post_detach('sub0B', 'sub0')",
        "_post_detach_children('sub0', ('sub0B',))",
        "_pre_attach_children('sub0', ('sub0',))",
        "_pre_detach_children('sub0', ())",
        "_post_detach_children('sub0', ())",
        "_pre_attach_children('sub0', ('sub0B',))",
        "_pre_attach('sub0B', 'sub0')",
        "_post_attach('sub0B', 'sub0')",
        "_post_attach_children('sub0', ('sub0B',))",
    ]
    TNode.TRACKING.clear()

    with assert_raises(LoopError, "Cannot set parent. TNode('/root/sub0') is parent of TNode('/root/sub0/sub0B')."):
        s0a.children = [s0]

    # test whether tree is unchanged after LoopError
    assert TNode.TRACKING == [
        "_pre_detach_children('sub0B', ())",
        "_post_detach_children('sub0B', ())",
        "_pre_attach_children('sub0B', ('sub0',))",
        "_pre_detach_children('sub0B', ())",
        "_post_detach_children('sub0B', ())",
        "_pre_attach_children('sub0B', ())",
        "_post_attach_children('sub0B', ())",
    ]
    TNode.TRACKING.clear()

    root.children = [s0, s1]

    assert TNode.TRACKING == [
        "_pre_detach_children('root', ('sub0', 'sub0A'))",
        "_pre_detach('sub0', 'root')",
        "_post_detach('sub0', 'root')",
        "_pre_detach('sub0A', 'root')",
        "_post_detach('sub0A', 'root')",
        "_post_detach_children('root', ('sub0', 'sub0A'))",
        "_pre_attach_children('root', ('sub0', 'sub0A'))",
        "_pre_attach('sub0', 'root')",
        "_post_attach('sub0', 'root')",
        "_pre_attach('sub0A', 'root')",
        "_post_attach('sub0A', 'root')",
        "_post_attach_children('root', ('sub0', 'sub0A'))",
    ]
    TNode.TRACKING.clear()

    s0.children = [s0a]

    assert TNode.TRACKING == [
        "_pre_detach_children('sub0', ('sub0B',))",
        "_pre_detach('sub0B', 'sub0')",
        "_post_detach('sub0B', 'sub0')",
        "_post_detach_children('sub0', ('sub0B',))",
        "_pre_attach_children('sub0', ('sub0B',))",
        "_pre_attach('sub0B', 'sub0')",
        "_post_attach('sub0B', 'sub0')",
        "_post_attach_children('sub0', ('sub0B',))",
    ]
    TNode.TRACKING.clear()

    s0a.children = [s1]

    assert TNode.TRACKING == [
        "_pre_detach_children('sub0B', ())",
        "_post_detach_children('sub0B', ())",
        "_pre_attach_children('sub0B', ('sub0A',))",
        "_pre_detach('sub0A', 'root')",
        "_post_detach('sub0A', 'root')",
        "_pre_attach('sub0A', 'sub0B')",
        "_post_attach('sub0A', 'sub0B')",
        "_post_attach_children('sub0B', ('sub0A',))",
    ]
    TNode.TRACKING.clear()

    root.children = [s0, s1]

    assert TNode.TRACKING == [
        "_pre_detach_children('root', ('sub0',))",
        "_pre_detach('sub0', 'root')",
        "_post_detach('sub0', 'root')",
        "_post_detach_children('root', ('sub0',))",
        "_pre_attach_children('root', ('sub0', 'sub0A'))",
        "_pre_attach('sub0', 'root')",
        "_post_attach('sub0', 'root')",
        "_pre_detach('sub0A', 'sub0B')",
        "_post_detach('sub0A', 'sub0B')",
        "_pre_attach('sub0A', 'root')",
        "_post_attach('sub0A', 'root')",
        "_post_attach_children('root', ('sub0', 'sub0A'))",
    ]
    TNode.TRACKING.clear()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html


# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = "anytree"
copyright = "2016-2023, c0fec0de"
author = "c0fec0de"
description = "Python Tree Data"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.doctest",
    "sphinx.ext.viewcode",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
]

templates_path = ["_templates"]
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

source_suffix = ".rst"

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinxdoc"
html_static_path = ["static"]

intersphinx_mapping = {
    "python": ("http://docs.python.org/3", None),
}
intersphinx_cache_limit = 1

autodoc_default_options = {
    "members": None,  # Include all members (methods).
    "member-order": "bysource",
}
autoclass_content = "both"



import codecs
import itertools
import re

import six

from anytree import PreOrderIter

_RE_ESC = re.compile(r'["\\]')


class MermaidExporter:

    """
    Mermaid Exporter.

    Args:
        node (Node): start node.

    Keyword Args:
        graph: Mermaid graph type.

        name: Mermaid graph name.

        options: list of options added to the graph.

        indent (int): number of spaces for indent.

        nodenamefunc: Function to extract node name from `node` object.
                      The function shall accept one `node` object as
                      argument and return the name of it.
                      Returns a unique identifier by default.

        nodefunc: Function to decorate a node with attributes.
                      The function shall accept one `node` object as
                      argument and return the attributes.
                      Returns ``[{node.name}]`` and creates therefore a
                      rectangular node by default.

        edgefunc: Function to decorate a edge with attributes.
                  The function shall accept two `node` objects as
                  argument. The first the node and the second the child
                  and return edge.
                  Returns ``-->`` by default.


        filter_: Function to filter nodes to include in export.
                 The function shall accept one `node` object as
                 argument and return True if it should be included,
                 or False if it should not be included.

        stop: stop iteration at `node` if `stop` function returns `True` for `node`.

        maxlevel (int): Limit export to this number of levels.

    >>> from anytree import Node
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root, edge=2)
    >>> s0b = Node("sub0B", parent=s0, foo=4, edge=109)
    >>> s0a = Node("sub0A", parent=s0, edge="")
    >>> s1 = Node("sub1", parent=root, edge="")
    >>> s1a = Node("sub1A", parent=s1, edge=7)
    >>> s1b = Node("sub1B", parent=s1, edge=8)
    >>> s1c = Node("sub1C", parent=s1, edge=22)
    >>> s1ca = Node("sub1Ca", parent=s1c, edge=42)

    A top-down graph:

    >>> from anytree.exporter import MermaidExporter
    >>> for line in MermaidExporter(root):
    ...     print(line)
    graph TD
    N0["root"]
    N1["sub0"]
    N2["sub0B"]
    N3["sub0A"]
    N4["sub1"]
    N5["sub1A"]
    N6["sub1B"]
    N7["sub1C"]
    N8["sub1Ca"]
    N0-->N1
    N0-->N4
    N1-->N2
    N1-->N3
    N4-->N5
    N4-->N6
    N4-->N7
    N7-->N8

    A customized graph with round boxes and named arrows:

    >>> def nodefunc(node):
    ...     return '("%s")' % (node.name)
    >>> def edgefunc(node, child):
    ...     return f"--{child.edge}-->"
    >>> options = [
    ...     "%% just an example comment",
    ...     "%% could be an option too",
    ... ]
    >>> for line in MermaidExporter(root, options=options, nodefunc=nodefunc, edgefunc=edgefunc):
    ...     print(line)
    graph TD
    %% just an example comment
    %% could be an option too
    N0("root")
    N1("sub0")
    N2("sub0B")
    N3("sub0A")
    N4("sub1")
    N5("sub1A")
    N6("sub1B")
    N7("sub1C")
    N8("sub1Ca")
    N0--2-->N1
    N0---->N4
    N1--109-->N2
    N1---->N3
    N4--7-->N5
    N4--8-->N6
    N4--22-->N7
    N7--42-->N8
    """

    def __init__(
        self,
        node,
        graph="graph",
        name="TD",
        options=None,
        indent=0,
        nodenamefunc=None,
        nodefunc=None,
        edgefunc=None,
        filter_=None,
        stop=None,
        maxlevel=None,
    ):
        self.node = node
        self.graph = graph
        self.name = name
        self.options = options
        self.indent = indent
        self.nodenamefunc = nodenamefunc
        self.nodefunc = nodefunc
        self.edgefunc = edgefunc
        self.filter_ = filter_
        self.stop = stop
        self.maxlevel = maxlevel
        self.__node_ids = {}
        self.__node_counter = itertools.count()

    def __iter__(self):
        # prepare
        indent = " " * self.indent
        nodenamefunc = self.nodenamefunc or self._default_nodenamefunc
        nodefunc = self.nodefunc or self._default_nodefunc
        edgefunc = self.edgefunc or self._default_edgefunc
        filter_ = self.filter_ or (lambda node: True)
        stop = self.stop or (lambda node: False)
        return self.__iter(indent, nodenamefunc, nodefunc, edgefunc, filter_, stop)

    # pylint: disable=arguments-differ
    def _default_nodenamefunc(self, node):
        node_id = id(node)
        try:
            num = self.__node_ids[node_id]
        except KeyError:
            num = self.__node_ids[node_id] = next(self.__node_counter)
        return "N%d" % (num,)

    @staticmethod
    def _default_nodefunc(node):
        # pylint: disable=W0613
        return '["%s"]' % (MermaidExporter.esc(node.name),)

    @staticmethod
    def _default_edgefunc(node, child):
        # pylint: disable=W0613
        return "-->"

    def __iter(self, indent, nodenamefunc, nodefunc, edgefunc, filter_, stop):
        yield "{self.graph} {self.name}".format(self=self)
        for option in self.__iter_options(indent):
            yield option
        for node in self.__iter_nodes(indent, nodenamefunc, nodefunc, filter_, stop):
            yield node
        for edge in self.__iter_edges(indent, nodenamefunc, edgefunc, filter_, stop):
            yield edge

    def __iter_options(self, indent):
        options = self.options
        if options:
            for option in options:
                yield "%s%s" % (indent, option)

    def __iter_nodes(self, indent, nodenamefunc, nodefunc, filter_, stop):
        for node in PreOrderIter(self.node, filter_=filter_, stop=stop, maxlevel=self.maxlevel):
            nodename = nodenamefunc(node)
            node = nodefunc(node)
            yield "%s%s%s" % (indent, nodename, node)

    def __iter_edges(self, indent, nodenamefunc, edgefunc, filter_, stop):
        maxlevel = self.maxlevel - 1 if self.maxlevel else None
        for node in PreOrderIter(self.node, filter_=filter_, stop=stop, maxlevel=maxlevel):
            nodename = nodenamefunc(node)
            for child in node.children:
                if filter_(child) and not stop(child):
                    childname = nodenamefunc(child)
                    edge = edgefunc(node, child)
                    yield "%s%s%s%s" % (
                        indent,
                        nodename,
                        edge,
                        childname,
                    )

    def to_file(self, filename):
        """
        Write graph to `filename`.

        >>> from anytree import Node
        >>> root = Node("root")
        >>> s0 = Node("sub0", parent=root)
        >>> s0b = Node("sub0B", parent=s0)
        >>> s0a = Node("sub0A", parent=s0)
        >>> s1 = Node("sub1", parent=root)
        >>> s1a = Node("sub1A", parent=s1)
        >>> s1b = Node("sub1B", parent=s1)
        >>> s1c = Node("sub1C", parent=s1)
        >>> s1ca = Node("sub1Ca", parent=s1c)

        >>> from anytree.exporter import MermaidExporter
        >>> MermaidExporter(root).to_file("tree.md")
        """
        with codecs.open(filename, "w", "utf-8") as file:
            file.write("```mermaid\n")
            for line in self:
                file.write("%s\n" % line)
            file.write("```")

    @staticmethod
    def esc(value):
        """Escape Strings."""
        return _RE_ESC.sub(lambda m: r"\%s" % m.group(0), six.text_type(value))



import codecs
import itertools
import logging
import re
from os import path, remove
from subprocess import check_call
from tempfile import NamedTemporaryFile

import six

from anytree import PreOrderIter

_RE_ESC = re.compile(r'["\\]')


class DotExporter:
    """
    Dot Language Exporter.

    Args:
        node (Node): start node.

    Keyword Args:
        graph: DOT graph type.

        name: DOT graph name.

        options: list of options added to the graph.

        indent (int): number of spaces for indent.

        nodenamefunc: Function to extract node name from `node` object.
                      The function shall accept one `node` object as
                      argument and return the name of it.

        nodeattrfunc: Function to decorate a node with attributes.
                      The function shall accept one `node` object as
                      argument and return the attributes.

        edgeattrfunc: Function to decorate a edge with attributes.
                      The function shall accept two `node` objects as
                      argument. The first the node and the second the child
                      and return the attributes.

        edgetypefunc: Function to which gives the edge type.
                      The function shall accept two `node` objects as
                      argument. The first the node and the second the child
                      and return the edge (i.e. '->').

        filter_: Function to filter nodes to include in export.
                 The function shall accept one `node` object as
                 argument and return True if it should be included,
                 or False if it should not be included.

        stop: stop iteration at `node` if `stop` function returns `True` for `node`.

        maxlevel (int): Limit export to this number of levels.

    >>> from anytree import Node
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root, edge=2)
    >>> s0b = Node("sub0B", parent=s0, foo=4, edge=109)
    >>> s0a = Node("sub0A", parent=s0, edge="")
    >>> s1 = Node("sub1", parent=root, edge="")
    >>> s1a = Node("sub1A", parent=s1, edge=7)
    >>> s1b = Node("sub1B", parent=s1, edge=8)
    >>> s1c = Node("sub1C", parent=s1, edge=22)
    >>> s1ca = Node("sub1Ca", parent=s1c, edge=42)

    .. note:: If the node names are not unqiue, see :any:`UniqueDotExporter`.

    A directed graph:

    >>> from anytree.exporter import DotExporter
    >>> for line in DotExporter(root):
    ...     print(line)
    digraph tree {
        "root";
        "sub0";
        "sub0B";
        "sub0A";
        "sub1";
        "sub1A";
        "sub1B";
        "sub1C";
        "sub1Ca";
        "root" -> "sub0";
        "root" -> "sub1";
        "sub0" -> "sub0B";
        "sub0" -> "sub0A";
        "sub1" -> "sub1A";
        "sub1" -> "sub1B";
        "sub1" -> "sub1C";
        "sub1C" -> "sub1Ca";
    }

    The resulting graph:

    .. image:: ../static/dotexporter0.png

    An undirected graph:

    >>> def nodenamefunc(node):
    ...     return '%s:%s' % (node.name, node.depth)
    >>> def edgeattrfunc(node, child):
    ...     return 'label="%s:%s"' % (node.name, child.name)
    >>> def edgetypefunc(node, child):
    ...     return '--'
            >>> from anytree.exporter import DotExporter
    >>> for line in DotExporter(root, graph="graph",
    ...                             nodenamefunc=nodenamefunc,
    ...                             nodeattrfunc=lambda node: "shape=box",
    ...                             edgeattrfunc=edgeattrfunc,
    ...                             edgetypefunc=edgetypefunc):
    ...     print(line)
    graph tree {
        "root:0" [shape=box];
        "sub0:1" [shape=box];
        "sub0B:2" [shape=box];
        "sub0A:2" [shape=box];
        "sub1:1" [shape=box];
        "sub1A:2" [shape=box];
        "sub1B:2" [shape=box];
        "sub1C:2" [shape=box];
        "sub1Ca:3" [shape=box];
        "root:0" -- "sub0:1" [label="root:sub0"];
        "root:0" -- "sub1:1" [label="root:sub1"];
        "sub0:1" -- "sub0B:2" [label="sub0:sub0B"];
        "sub0:1" -- "sub0A:2" [label="sub0:sub0A"];
        "sub1:1" -- "sub1A:2" [label="sub1:sub1A"];
        "sub1:1" -- "sub1B:2" [label="sub1:sub1B"];
        "sub1:1" -- "sub1C:2" [label="sub1:sub1C"];
        "sub1C:2" -- "sub1Ca:3" [label="sub1C:sub1Ca"];
    }

    The resulting graph:

    .. image:: ../static/dotexporter1.png

    To export custom node implementations or :any:`AnyNode`, please provide a proper `nodenamefunc`:

    >>> from anytree import AnyNode
    >>> root = AnyNode(id="root")
    >>> s0 = AnyNode(id="sub0", parent=root)
    >>> s0b = AnyNode(id="s0b", parent=s0)
    >>> s0a = AnyNode(id="s0a", parent=s0)

    >>> from anytree.exporter import DotExporter
    >>> for line in DotExporter(root, nodenamefunc=lambda n: n.id):
    ...     print(line)
    digraph tree {
        "root";
        "sub0";
        "s0b";
        "s0a";
        "root" -> "sub0";
        "sub0" -> "s0b";
        "sub0" -> "s0a";
    }
    """

    def __init__(
        self,
        node,
        graph="digraph",
        name="tree",
        options=None,
        indent=4,
        nodenamefunc=None,
        nodeattrfunc=None,
        edgeattrfunc=None,
        edgetypefunc=None,
        filter_=None,
        maxlevel=None,
        stop=None,
    ):
        self.node = node
        self.graph = graph
        self.name = name
        self.options = options
        self.indent = indent
        self.nodenamefunc = nodenamefunc
        self.nodeattrfunc = nodeattrfunc
        self.edgeattrfunc = edgeattrfunc
        self.edgetypefunc = edgetypefunc
        self.filter_ = filter_
        self.maxlevel = maxlevel
        self.stop = stop

    def __iter__(self):
        # prepare
        indent = " " * self.indent
        nodenamefunc = self.nodenamefunc or self._default_nodenamefunc
        nodeattrfunc = self.nodeattrfunc or self._default_nodeattrfunc
        edgeattrfunc = self.edgeattrfunc or self._default_edgeattrfunc
        edgetypefunc = self.edgetypefunc or self._default_edgetypefunc
        filter_ = self.filter_ or self._default_filter
        return self.__iter(indent, nodenamefunc, nodeattrfunc, edgeattrfunc, edgetypefunc, filter_)

    @staticmethod
    def _default_nodenamefunc(node):
        return node.name

    @staticmethod
    def _default_nodeattrfunc(node):
        # pylint: disable=unused-argument
        return None

    @staticmethod
    def _default_edgeattrfunc(node, child):
        # pylint: disable=unused-argument
        return None

    @staticmethod
    def _default_edgetypefunc(node, child):
        # pylint: disable=unused-argument
        return "->"

    @staticmethod
    def _default_filter(node):
        # pylint: disable=unused-argument
        return True

    def __iter(self, indent, nodenamefunc, nodeattrfunc, edgeattrfunc, edgetypefunc, filter_):
        yield "{self.graph} {self.name} {{".format(self=self)
        for option in self.__iter_options(indent):
            yield option
        for node in self.__iter_nodes(indent, nodenamefunc, nodeattrfunc, filter_):
            yield node
        for edge in self.__iter_edges(indent, nodenamefunc, edgeattrfunc, edgetypefunc, filter_):
            yield edge
        yield "}"

    def __iter_options(self, indent):
        options = self.options
        if options:
            for option in options:
                yield "%s%s" % (indent, option)

    def __iter_nodes(self, indent, nodenamefunc, nodeattrfunc, filter_):
        for node in PreOrderIter(self.node, filter_=filter_, stop=self.stop, maxlevel=self.maxlevel):
            nodename = nodenamefunc(node)
            nodeattr = nodeattrfunc(node)
            nodeattr = " [%s]" % nodeattr if nodeattr is not None else ""
            yield '%s"%s"%s;' % (indent, DotExporter.esc(nodename), nodeattr)

    def __iter_edges(self, indent, nodenamefunc, edgeattrfunc, edgetypefunc, filter_):
        maxlevel = self.maxlevel - 1 if self.maxlevel else None
        for node in PreOrderIter(self.node, filter_=filter_, stop=self.stop, maxlevel=maxlevel):
            nodename = nodenamefunc(node)
            for child in node.children:
                if not filter_(child):
                    continue
                childname = nodenamefunc(child)
                edgeattr = edgeattrfunc(node, child)
                edgetype = edgetypefunc(node, child)
                edgeattr = " [%s]" % edgeattr if edgeattr is not None else ""
                yield '%s"%s" %s "%s"%s;' % (
                    indent,
                    DotExporter.esc(nodename),
                    edgetype,
                    DotExporter.esc(childname),
                    edgeattr,
                )

    def to_dotfile(self, filename):
        """
        Write graph to `filename`.

        >>> from anytree import Node
        >>> root = Node("root")
        >>> s0 = Node("sub0", parent=root)
        >>> s0b = Node("sub0B", parent=s0)
        >>> s0a = Node("sub0A", parent=s0)
        >>> s1 = Node("sub1", parent=root)
        >>> s1a = Node("sub1A", parent=s1)
        >>> s1b = Node("sub1B", parent=s1)
        >>> s1c = Node("sub1C", parent=s1)
        >>> s1ca = Node("sub1Ca", parent=s1c)

        >>> from anytree.exporter import DotExporter
        >>> DotExporter(root).to_dotfile("tree.dot")

        The generated file should be handed over to the `dot` tool from the
        http://www.graphviz.org/ package::

            $ dot tree.dot -T png -o tree.png
        """
        with codecs.open(filename, "w", "utf-8") as file:
            for line in self:
                file.write("%s\n" % line)

    def to_picture(self, filename):
        """
        Write graph to a temporary file and invoke `dot`.

        The output file type is automatically detected from the file suffix.

        *`graphviz` needs to be installed, before usage of this method.*
        """
        fileformat = path.splitext(filename)[1][1:]
        with NamedTemporaryFile("wb", delete=False) as dotfile:
            dotfilename = dotfile.name
            for line in self:
                dotfile.write(("%s\n" % line).encode("utf-8"))
            dotfile.flush()
            cmd = ["dot", dotfilename, "-T", fileformat, "-o", filename]
            check_call(cmd)
        try:
            remove(dotfilename)
        # pylint: disable=broad-exception-caught
        except Exception:  # pragma: no cover
            logging.getLogger(__name__).warning("Could not remove temporary file %s", dotfilename)

    @staticmethod
    def esc(value):
        """Escape Strings."""
        return _RE_ESC.sub(lambda m: r"\%s" % m.group(0), six.text_type(value))


class UniqueDotExporter(DotExporter):
    """
    Unqiue Dot Language Exporter.

    Handle trees with random or conflicting node names gracefully.

    Args:
        node (Node): start node.

    Keyword Args:
        graph: DOT graph type.

        name: DOT graph name.

        options: list of options added to the graph.

        indent (int): number of spaces for indent.

        nodenamefunc: Function to extract node name from `node` object.
                        The function shall accept one `node` object as
                        argument and return the name of it.

        nodeattrfunc: Function to decorate a node with attributes.
                        The function shall accept one `node` object as
                        argument and return the attributes.

        edgeattrfunc: Function to decorate a edge with attributes.
                        The function shall accept two `node` objects as
                        argument. The first the node and the second the child
                        and return the attributes.

        edgetypefunc: Function to which gives the edge type.
                        The function shall accept two `node` objects as
                        argument. The first the node and the second the child
                        and return the edge (i.e. '->').

        filter_: Function to filter nodes to include in export.
                 The function shall accept one `node` object as
                 argument and return True if it should be included,
                 or False if it should not be included.

        stop: stop iteration at `node` if `stop` function returns `True` for `node`.

        maxlevel (int): Limit export to this number of levels.

    >>> from anytree import Node
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root)
    >>> s0b = Node("s0", parent=s0)
    >>> s0a = Node("s0", parent=s0)
    >>> s1 = Node("sub1", parent=root)
    >>> s1a = Node("s1", parent=s1)
    >>> s1b = Node("s1", parent=s1)
    >>> s1c = Node("s1", parent=s1)
    >>> s1ca = Node("sub1Ca", parent=s1c)

    >>> from anytree.exporter import UniqueDotExporter
    >>> for line in UniqueDotExporter(root):
    ...     print(line)
    digraph tree {
        "0x0" [label="root"];
        "0x1" [label="sub0"];
        "0x2" [label="s0"];
        "0x3" [label="s0"];
        "0x4" [label="sub1"];
        "0x5" [label="s1"];
        "0x6" [label="s1"];
        "0x7" [label="s1"];
        "0x8" [label="sub1Ca"];
        "0x0" -> "0x1";
        "0x0" -> "0x4";
        "0x1" -> "0x2";
        "0x1" -> "0x3";
        "0x4" -> "0x5";
        "0x4" -> "0x6";
        "0x4" -> "0x7";
        "0x7" -> "0x8";
    }

    The resulting graph:

    .. image:: ../static/uniquedotexporter2.png

    To export custom node implementations or :any:`AnyNode`, please provide a proper `nodeattrfunc`:

    >>> from anytree import AnyNode
    >>> root = AnyNode(id="root")
    >>> s0 = AnyNode(id="sub0", parent=root)
    >>> s0b = AnyNode(id="s0", parent=s0)
    >>> s0a = AnyNode(id="s0", parent=s0)

    >>> from anytree.exporter import UniqueDotExporter
    >>> for line in UniqueDotExporter(root, nodeattrfunc=lambda n: 'label="%s"' % (n.id)):
    ...     print(line)
    digraph tree {
        "0x0" [label="root"];
        "0x1" [label="sub0"];
        "0x2" [label="s0"];
        "0x3" [label="s0"];
        "0x0" -> "0x1";
        "0x1" -> "0x2";
        "0x1" -> "0x3";
    }
    """

    def __init__(
        self,
        node,
        graph="digraph",
        name="tree",
        options=None,
        indent=4,
        nodenamefunc=None,
        nodeattrfunc=None,
        edgeattrfunc=None,
        edgetypefunc=None,
        filter_=None,
        stop=None,
        maxlevel=None,
    ):
        super(UniqueDotExporter, self).__init__(
            node,
            graph=graph,
            name=name,
            options=options,
            indent=indent,
            nodenamefunc=nodenamefunc,
            nodeattrfunc=nodeattrfunc,
            edgeattrfunc=edgeattrfunc,
            edgetypefunc=edgetypefunc,
            filter_=filter_,
            stop=stop,
            maxlevel=maxlevel,
        )
        self.__node_ids = {}
        self.__node_counter = itertools.count()

    # pylint: disable=arguments-differ
    def _default_nodenamefunc(self, node):
        node_id = id(node)
        try:
            num = self.__node_ids[node_id]
        except KeyError:
            num = self.__node_ids[node_id] = next(self.__node_counter)
        return hex(num)

    @staticmethod
    def _default_nodeattrfunc(node):
        return 'label="%s"' % (node.name,)



# -*- coding: utf-8 -*-
from copy import deepcopy

from anytree import Node, RenderTree
from anytree.exporter import DictExporter
from anytree.importer import DictImporter

from .helper import eq_


def test_dict_importer():
    """Dict Importer."""
    importer = DictImporter()
    exporter = DictExporter()
    refdata = {
        "id": "root",
        "children": [
            {"id": "sub0", "children": [{"id": "sub0B"}, {"id": "sub0A"}]},
            {
                "id": "sub1",
                "children": [{"id": "sub1A"}, {"id": "sub1B"}, {"id": "sub1C", "children": [{"id": "sub1Ca"}]}],
            },
        ],
    }
    data = deepcopy(refdata)
    root = importer.import_(data)
    eq_(data, refdata)
    eq_(exporter.export(root), data)
    r = RenderTree(root)
    assert str(r).splitlines() == [
        "AnyNode(id='root')",
        "├── AnyNode(id='sub0')",
        "│   ├── AnyNode(id='sub0B')",
        "│   └── AnyNode(id='sub0A')",
        "└── AnyNode(id='sub1')",
        "    ├── AnyNode(id='sub1A')",
        "    ├── AnyNode(id='sub1B')",
        "    └── AnyNode(id='sub1C')",
        "        └── AnyNode(id='sub1Ca')",
    ]


def test_dict_importer_node():
    """Dict Importer."""
    importer = DictImporter(Node)
    exporter = DictExporter()
    refdata = {
        "name": "root",
        "children": [
            {"name": "sub0", "children": [{"name": "sub0B"}, {"name": "sub0A"}]},
            {
                "name": "sub1",
                "children": [
                    {"name": "sub1A"},
                    {"name": "sub1B"},
                    {"name": "sub1C", "children": [{"name": "sub1Ca"}]},
                ],
            },
        ],
    }
    data = deepcopy(refdata)
    root = importer.import_(data)
    eq_(data, refdata)
    eq_(exporter.export(root), data)
    r = RenderTree(root)
    assert str(r).splitlines() == [
        "Node('/root')",
        "├── Node('/root/sub0')",
        "│   ├── Node('/root/sub0/sub0B')",
        "│   └── Node('/root/sub0/sub0A')",
        "└── Node('/root/sub1')",
        "    ├── Node('/root/sub1/sub1A')",
        "    ├── Node('/root/sub1/sub1B')",
        "    └── Node('/root/sub1/sub1C')",
        "        └── Node('/root/sub1/sub1C/sub1Ca')",
    ]



# -*- coding: utf-8 -*-
from anytree import LoopError, Node, NodeMixin, PostOrderIter, PreOrderIter, TreeError

from .helper import assert_raises, eq_


def test_readonly_pre():
    """Read Only Use case, where Exceptions in _pre_{attach,detach} avoid modifications."""

    class ReadonlyError(RuntimeError):
        pass

    class ReadonlyNode(Node):

        _is_readonly = False

        def _pre_attach(self, parent):
            if self._is_readonly:
                raise ReadonlyError()

        def _pre_detach(self, parent):
            if self._is_readonly:
                raise ReadonlyError()

    # construct and make readonly!
    root = ReadonlyNode("root")
    s0 = ReadonlyNode("sub0", parent=root)
    s0b = ReadonlyNode("sub0B", parent=s0)
    s0a = ReadonlyNode("sub0A", parent=s0)
    s1 = ReadonlyNode("sub1", parent=root)
    s1a = ReadonlyNode("sub1A", parent=s1)
    s1b = ReadonlyNode("sub1B", parent=s1)
    s1c = ReadonlyNode("sub1C", parent=s1)
    s1ca = ReadonlyNode("sub1Ca", parent=s1c)
    ReadonlyNode._is_readonly = True

    def check():
        eq_(root.parent, None)
        eq_(root.children, tuple([s0, s1]))
        eq_(s0.parent, root)
        eq_(s0.children, tuple([s0b, s0a]))
        eq_(s0b.parent, s0)
        eq_(s0b.children, tuple())
        eq_(s0a.parent, s0)
        eq_(s0a.children, tuple())
        eq_(s1.parent, root)
        eq_(s1.children, tuple([s1a, s1b, s1c]))
        eq_(s1a.parent, s1)
        eq_(s1a.children, tuple())
        eq_(s1b.parent, s1)
        eq_(s1b.children, tuple())
        eq_(s1c.parent, s1)
        eq_(s1c.children, tuple([s1ca]))
        eq_(s1ca.parent, s1c)
        eq_(s1ca.children, tuple())

    check()
    with assert_raises(ReadonlyError, ""):
        s1ca.parent = s0
    check()
    with assert_raises(ReadonlyError, ""):
        s1ca.parent = None
    check()
    with assert_raises(ReadonlyError, ""):
        s0.children = []
    check()



# -*- coding: utf-8 -*-
from anytree import AnyNode, Node, NodeMixin, PostOrderIter, PreOrderIter, SymlinkNode

from .helper import eq_


def test_symlink():

    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root, foo=4)
    s1a = Node("sub1A", parent=s1)
    s1b = Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)
    ln = SymlinkNode(s1, parent=root, blub=17)
    l0 = Node("l0", parent=ln)

    eq_(root.parent, None)
    eq_(root.children, tuple([s0, s1, ln]))
    eq_(s0.parent, root)
    eq_(s0.children, tuple([s0b, s0a]))
    eq_(s0b.parent, s0)
    eq_(s0b.children, tuple())
    eq_(s0a.parent, s0)
    eq_(s0a.children, tuple())
    eq_(s1.parent, root)
    eq_(s1.children, tuple([s1a, s1b, s1c]))
    eq_(s1.foo, 4)
    eq_(s1a.parent, s1)
    eq_(s1a.children, tuple())
    eq_(s1b.parent, s1)
    eq_(s1b.children, tuple())
    eq_(s1c.parent, s1)
    eq_(s1c.children, tuple([s1ca]))
    eq_(s1ca.parent, s1c)
    eq_(s1ca.children, tuple())
    eq_(ln.parent, root)
    eq_(ln.children, tuple([l0]))
    eq_(ln.foo, 4)

    eq_(s1.blub, 17)
    eq_(ln.blub, 17)

    ln.bar = 9
    eq_(ln.bar, 9)
    eq_(s1.bar, 9)

    result = [node.name for node in PreOrderIter(root)]
    eq_(result, ["root", "sub0", "sub0B", "sub0A", "sub1", "sub1A", "sub1B", "sub1C", "sub1Ca", "sub1", "l0"])

    result = [node.name for node in PostOrderIter(root)]
    eq_(result, ["sub0B", "sub0A", "sub0", "sub1A", "sub1B", "sub1Ca", "sub1C", "sub1", "l0", "sub1", "root"])



# -*- coding: utf-8 -*-

from .nodemixin import NodeMixin
from .util import _repr


class Node(NodeMixin):
    """
    A simple tree node with a `name` and any `kwargs`.

    Args:
        name: A name or any other object this node can reference to as identifier.

    Keyword Args:
        parent: Reference to parent node.
        children: Iterable with child nodes.
        *: Any other given attribute is just stored as object attribute.

    Other than :any:`AnyNode` this class has at least the `name` attribute,
    to distinguish between different instances.

    The `parent` attribute refers the parent node:

    >>> from anytree import Node, RenderTree
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root)
    >>> s0b = Node("sub0B", parent=s0, foo=4, bar=109)
    >>> s0a = Node("sub0A", parent=s0)
    >>> s1 = Node("sub1", parent=root)
    >>> s1a = Node("sub1A", parent=s1)
    >>> s1b = Node("sub1B", parent=s1, bar=8)
    >>> s1c = Node("sub1C", parent=s1)
    >>> s1ca = Node("sub1Ca", parent=s1c)

    >>> print(RenderTree(root))
    Node('/root')
    ├── Node('/root/sub0')
    │   ├── Node('/root/sub0/sub0B', bar=109, foo=4)
    │   └── Node('/root/sub0/sub0A')
    └── Node('/root/sub1')
        ├── Node('/root/sub1/sub1A')
        ├── Node('/root/sub1/sub1B', bar=8)
        └── Node('/root/sub1/sub1C')
            └── Node('/root/sub1/sub1C/sub1Ca')

    The same tree can be constructed by using the `children` attribute:

    >>> root = Node("root", children=[
    ...     Node("sub0", children=[
    ...         Node("sub0B", bar=109, foo=4),
    ...         Node("sub0A", children=None),
    ...     ]),
    ...     Node("sub1", children=[
    ...         Node("sub1A"),
    ...         Node("sub1B", bar=8, children=[]),
    ...         Node("sub1C", children=[
    ...             Node("sub1Ca"),
    ...         ]),
    ...     ]),
    ... ])

    >>> print(RenderTree(root))
    Node('/root')
    ├── Node('/root/sub0')
    │   ├── Node('/root/sub0/sub0B', bar=109, foo=4)
    │   └── Node('/root/sub0/sub0A')
    └── Node('/root/sub1')
        ├── Node('/root/sub1/sub1A')
        ├── Node('/root/sub1/sub1B', bar=8)
        └── Node('/root/sub1/sub1C')
            └── Node('/root/sub1/sub1C/sub1Ca')
    """

    def __init__(self, name, parent=None, children=None, **kwargs):
        self.__dict__.update(kwargs)
        self.name = name
        self.parent = parent
        if children:
            self.children = children

    def __repr__(self):
        args = ["%r" % self.separator.join([""] + [str(node.name) for node in self.path])]
        return _repr(self, args=args, nameblacklist=["name"])



# -*- coding: utf-8 -*-
from anytree import AnyNode, LoopError, Node, NodeMixin, PostOrderIter, PreOrderIter, TreeError

from .helper import assert_raises


def test_node_parent_error():
    """Node Parent Error."""
    with assert_raises(TreeError, "Parent node 'parent' is not of type 'NodeMixin'."):
        Node("root", "parent")


def test_parent_child():
    """A tree parent and child attributes."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1a = Node("sub1A", parent=s1)
    s1b = Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.parent is None
    assert root.children == tuple([s0, s1])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent == root
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple([s1ca])
    assert s1ca.parent == s1c
    assert s1ca.children == tuple()

    # change parent
    s1ca.parent = s0

    assert root.parent is None
    assert root.children == tuple([s0, s1])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a, s1ca])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent == root
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple()
    assert s1ca.parent == s0
    assert s1ca.children == tuple()

    # break tree into two
    s1.parent = None

    assert root.parent is None
    assert root.children == tuple([s0])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a, s1ca])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent is None
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple()
    assert s1ca.parent == s0
    assert s1ca.children == tuple()

    # set to the same
    s1b.parent = s1

    assert root.parent is None
    assert root.children == tuple([s0])
    assert s0.parent == root
    assert s0.children == tuple([s0b, s0a, s1ca])
    assert s0b.parent == s0
    assert s0b.children == tuple()
    assert s0a.parent == s0
    assert s0a.children == tuple()
    assert s1.parent is None
    assert s1.children == tuple([s1a, s1b, s1c])
    assert s1a.parent == s1
    assert s1a.children == tuple()
    assert s1b.parent == s1
    assert s1b.children == tuple()
    assert s1c.parent == s1
    assert s1c.children == tuple()
    assert s1ca.parent == s0
    assert s1ca.children == tuple()


def test_detach_children():

    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1a = Node("sub1A", parent=s1)
    s1b = Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.descendants == (s0, s0b, s0a, s1, s1a, s1b, s1c, s1ca)
    del s0.children
    assert root.descendants == (s0, s1, s1a, s1b, s1c, s1ca)
    del s1.children
    assert root.descendants == (s0, s1)


def test_children_setter():

    root = Node("root")
    s0 = Node("sub0")
    s1 = Node("sub0A")
    s0a = Node("sub0B")

    root.children = [s0, s1]
    s0.children = [s0a]
    assert root.descendants == (s0, s0a, s1)

    with assert_raises(LoopError, "Cannot set parent. Node('/root/sub0') cannot be parent of itself."):
        s0.children = [s0]

    # test whether tree is unchanged after LoopError
    assert root.descendants == (s0, s0a, s1)

    with assert_raises(LoopError, "Cannot set parent. Node('/root/sub0') is parent of Node('/root/sub0/sub0B')."):
        s0a.children = [s0]

    # test whether tree is unchanged after LoopError
    assert root.descendants == (s0, s0a, s1)

    root.children = [s0, s1]
    s0.children = [s0a]
    s0a.children = [s1]
    assert root.descendants == (s0, s0a, s1)


def test_children_setter_large():

    root = Node("root")
    s0 = Node("sub0")
    s0b = Node("sub0B")
    s0a = Node("sub0A")
    s1 = Node("sub1")
    s1a = Node("sub1A")
    s1b = Node("sub1B")
    s1c = Node("sub1C")
    s1ca = Node("sub1Ca")

    root.children = [s0, s1]
    assert root.descendants == (s0, s1)
    s0.children = [s0a, s0b]
    assert root.descendants == (s0, s0a, s0b, s1)
    s1.children = [s1a, s1b, s1c]
    assert root.descendants == (s0, s0a, s0b, s1, s1a, s1b, s1c)
    with assert_raises(TypeError, "'Node' object is not iterable"):
        s1.children = s1ca
    assert root.descendants == (s0, s0a, s0b, s1, s1a, s1b, s1c)


def test_node_children_type():

    root = Node("root")
    with assert_raises(TreeError, "Cannot add non-node object 'string'. It is not a subclass of 'NodeMixin'."):
        root.children = ["string"]


def test_node_children_multiple():

    root = Node("root")
    sub = Node("sub")
    with assert_raises(TreeError, "Cannot add node Node('/sub') multiple times as child."):
        root.children = [sub, sub]


def test_recursion_detection():
    """Recursion detection."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)

    # try recursion
    assert root.parent is None
    try:
        root.parent = root
    except LoopError as exc:
        assert str(exc) == "Cannot set parent. Node('/root') cannot be parent of itself."
        assert root.parent is None
    else:
        assert False

    assert root.parent is None
    try:
        root.parent = s0a
    except LoopError as exc:
        assert str(exc) == ("Cannot set parent. Node('/root') is parent of Node('/root/sub0/sub0A').")
        assert root.parent is None
    else:
        assert False

    assert s0.parent is root
    try:
        s0.parent = s0a
    except LoopError as exc:
        assert str(exc) == ("Cannot set parent. Node('/root/sub0') is parent of Node('/root/sub0/sub0A').")
        assert s0.parent is root
    else:
        assert False


def test_repr():
    """Node representation."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s1 = Node("sub1", parent=root, foo=42, bar="c0fe")

    assert repr(root) == "Node('/root')"
    assert repr(s0) == "Node('/root/sub0')"
    assert repr(s1) == "Node('/root/sub1', bar='c0fe', foo=42)"


def test_ancestors():
    """Node.ancestors."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.ancestors == tuple()
    assert s0.ancestors == tuple([root])
    assert s0b.ancestors == tuple([root, s0])
    assert s0a.ancestors == tuple([root, s0])
    assert s1ca.ancestors == tuple([root, s1, s1c])
    # deprecated typo
    assert s1ca.anchestors == tuple([root, s1, s1c])


def test_node_children_init():
    """Node With Children Attribute."""
    root = Node("root", children=[Node("a", children=[Node("aa")]), Node("b")])
    assert repr(root.descendants) == "(Node('/root/a'), Node('/root/a/aa'), Node('/root/b'))"


def test_anynode_children_init():
    """Anynode With Children Attribute."""
    root = AnyNode(foo="root", children=[AnyNode(foo="a", children=[AnyNode(foo="aa")]), AnyNode(foo="b")])
    assert repr(root.descendants) == "(AnyNode(foo='a'), AnyNode(foo='aa'), AnyNode(foo='b'))"


def test_descendants():
    """Node.descendants."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.descendants == tuple([s0, s0b, s0a, s1, s1c, s1ca])
    assert s1.descendants == tuple([s1c, s1ca])
    assert s1c.descendants == tuple([s1ca])
    assert s1ca.descendants == tuple()


def test_root():
    """Node.root."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.root == root
    assert s0.root == root
    assert s0b.root == root
    assert s0a.root == root
    assert s1.root == root
    assert s1c.root == root
    assert s1ca.root == root


def test_siblings():
    """Node.siblings."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.siblings == tuple()
    assert s0.siblings == tuple([s1])
    assert s0b.siblings == tuple([s0a])
    assert s0a.siblings == tuple([s0b])
    assert s1.siblings == tuple([s0])
    assert s1c.siblings == tuple()
    assert s1ca.siblings == tuple()


def test_is_leaf():
    """Node.is_leaf."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.is_leaf is False
    assert s0.is_leaf is False
    assert s0b.is_leaf is True
    assert s0a.is_leaf is True
    assert s1.is_leaf is False
    assert s1c.is_leaf is False
    assert s1ca.is_leaf is True


def test_leaves():
    """Node.leaves."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.leaves == tuple([s0b, s0a, s1ca])
    assert s0.leaves == tuple([s0b, s0a])
    assert s0b.leaves == tuple([s0b])
    assert s0a.leaves == tuple([s0a])
    assert s1.leaves == tuple([s1ca])
    assert s1c.leaves == tuple([s1ca])
    assert s1ca.leaves == tuple([s1ca])


def test_is_root():
    """Node.is_root."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.is_root is True
    assert s0.is_root is False
    assert s0b.is_root is False
    assert s0a.is_root is False
    assert s1.is_root is False
    assert s1c.is_root is False
    assert s1ca.is_root is False


def test_height():
    """Node.height."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.height == 3
    assert s0.height == 1
    assert s0b.height == 0
    assert s0a.height == 0
    assert s1.height == 2
    assert s1c.height == 1
    assert s1ca.height == 0


def test_size():
    """Node.size."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.size == 7
    assert s0.size == 3
    assert s0b.size == 1
    assert s0a.size == 1
    assert s1.size == 3
    assert s1c.size == 2
    assert s1ca.size == 1


def test_size():
    """Node.size."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.size == 7
    assert s0.size == 3
    assert s0b.size == 1
    assert s0a.size == 1
    assert s1.size == 3
    assert s1c.size == 2
    assert s1ca.size == 1


def test_depth():
    """Node.depth."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    assert root.depth == 0
    assert s0.depth == 1
    assert s0b.depth == 2
    assert s0a.depth == 2
    assert s1.depth == 1
    assert s1c.depth == 2
    assert s1ca.depth == 3


def test_parent():
    """Parent attribute."""
    foo = NodeMixin()
    assert foo.parent is None


def test_pre_order_iter():
    """Pre-Order Iterator."""
    f = Node("f")
    b = Node("b", parent=f)
    a = Node("a", parent=b)
    d = Node("d", parent=b)
    c = Node("c", parent=d)
    e = Node("e", parent=d)
    g = Node("g", parent=f)
    i = Node("i", parent=g)
    h = Node("h", parent=i)

    assert [node.name for node in PreOrderIter(f)] == ["f", "b", "a", "d", "c", "e", "g", "i", "h"]


def test_post_order_iter():
    """Post-Order Iterator."""
    f = Node("f")
    b = Node("b", parent=f)
    a = Node("a", parent=b)
    d = Node("d", parent=b)
    c = Node("c", parent=d)
    e = Node("e", parent=d)
    g = Node("g", parent=f)
    i = Node("i", parent=g)
    h = Node("h", parent=i)

    assert [node.name for node in PostOrderIter(f)] == ["a", "c", "e", "d", "b", "h", "i", "g", "f"]


def test_anyname():
    """Support any type as name."""
    myroot = Node([1, 2, 3])
    Node("/foo", parent=myroot)
    assert str(myroot) == "Node('/[1, 2, 3]')"


def test_node_kwargs():
    """Ticket #24."""

    class MyNode(Node):
        def __init__(self, name, parent=None, **kwargs):
            super(MyNode, self).__init__(name, parent, **kwargs)

        def _post_attach(self, parent):
            print(self.my_attribute)

    node_a = MyNode("A")
    node_b = MyNode("B", node_a, my_attribute=True)
    assert repr(node_b) == "MyNode('/A/B', my_attribute=True)"


def test_hookups():
    """Hookup attributes #29."""

    class MyNode(Node):
        def _pre_attach(self, parent):
            assert str(self.parent) == "None"
            assert self.children == tuple()
            assert str(self.path) == "(MyNode('/B'),)"

        def _post_attach(self, parent):
            assert str(self.parent) == "MyNode('/A')"
            assert self.children == tuple()
            assert str(self.path) == "(MyNode('/A'), MyNode('/A/B'))"

        def _pre_detach(self, parent):
            assert str(self.parent) == "MyNode('/A')"
            assert self.children == tuple()
            assert str(self.path) == "(MyNode('/A'), MyNode('/A/B'))"

        def _post_detach(self, parent):
            assert str(self.parent) == "None"
            assert self.children == tuple()
            assert str(self.path) == "(MyNode('/B'),)"

    node_a = MyNode("A")
    node_b = MyNode("B", node_a)  # attach B on A
    node_b.parent = None  # detach B from A


def test_any_node_parent_error():
    """Any Node Parent Error."""

    with assert_raises(TreeError, "Parent node 'r' is not of type 'NodeMixin'."):
        AnyNode("r")


def test_any_node():
    """Any Node."""
    r = AnyNode()
    a = AnyNode()
    b = AnyNode(foo=4)
    assert r.parent is None
    assert a.parent is None
    assert b.parent is None
    a.parent = r
    b.parent = r
    assert r.children == (a, b)
    assert repr(r) == "AnyNode()"
    assert repr(a) == "AnyNode()"
    assert repr(b) == "AnyNode(foo=4)"


def test_eq_overwrite():
    """Node with overwritten __eq__."""

    class EqOverwrittingNode(NodeMixin):
        def __init__(self, a, b, parent=None):
            super(EqOverwrittingNode, self).__init__()
            self.a = a
            self.b = b
            self.parent = parent

        def __eq__(self, other):
            if isinstance(other, EqOverwrittingNode):
                return self.a == other.a and self.b == other.b
            else:
                return NotImplemented

    r = EqOverwrittingNode(0, 0)
    a = EqOverwrittingNode(1, 0, parent=r)
    b = EqOverwrittingNode(1, 0, parent=r)
    assert a.parent is r
    assert b.parent is r
    assert a.a == 1
    assert a.b == 0
    assert b.a == 1
    assert b.b == 0


def test_tuple():
    """Tuple as parent."""
    with assert_raises(TreeError, "Parent node (1, 0, 3) is not of type 'NodeMixin'."):
        Node((0, 1, 2), parent=(1, 0, 3))


def test_tuple_as_children():
    """Tuple as children."""
    n = Node("foo")
    with assert_raises(TreeError, "Cannot add non-node object (0, 1, 2). It is not a subclass of 'NodeMixin'."):
        n.children = [(0, 1, 2)]



# -*- coding: utf-8 -*-
import pathlib

from pytest import fixture

from anytree import Node
from anytree.exporter import UniqueDotExporter

REFDATA = pathlib.Path(__file__).parent / "refdata" / "test_uniquedotexporter"


from .util import assert_gen


@fixture
def root():
    root = Node("root")
    s0 = Node("sub0", parent=root, edge=2)
    Node("sub0B", parent=s0, foo=4, edge=109)
    Node("sub0A", parent=s0, edge="")
    s1 = Node("sub1", parent=root, edge="")
    Node("sub1A", parent=s1, edge=7)
    Node('sub1"B', parent=s1, edge=8)
    s1c = Node("su\\b1C", parent=s1, edge=22)
    Node("sub1Ca", parent=s1c, edge=42)
    yield root


def test_tree(tmp_path, root):
    """Tree."""
    UniqueDotExporter(root).to_dotfile(tmp_path / "tree.dot")
    assert_gen(tmp_path, REFDATA / "tree")


def test_tree_custom(tmp_path, root):
    """Tree Custom."""

    def nodenamefunc(node):
        return "%s:%s" % (node.name, node.depth)

    def edgeattrfunc(node, child):
        return 'label="%s:%s"' % (node.name, child.name)

    def nodefunc(node):
        return '("%s")' % (node.name)

    def edgefunc(node, child):
        return f"--{child.edge}-->"

    UniqueDotExporter(
        root,
        options=["rankdir=LR;"],
        nodenamefunc=nodenamefunc,
        nodeattrfunc=lambda node: "shape=box",
        edgeattrfunc=edgeattrfunc,
    ).to_dotfile(tmp_path / "tree_custom.dot")
    assert_gen(tmp_path, REFDATA / "tree_custom")


def test_tree_filter(tmp_path, root):
    """Tree with Filter."""
    UniqueDotExporter(root, filter_=lambda node: node.name.startswith("sub")).to_dotfile(tmp_path / "tree_filter.dot")
    assert_gen(tmp_path, REFDATA / "tree_filter")


def test_tree_stop(tmp_path, root):
    """Tree with stop."""
    UniqueDotExporter(root, stop=lambda node: node.name == "sub1").to_dotfile(tmp_path / "tree_stop.dot")
    assert_gen(tmp_path, REFDATA / "tree_stop")


def test_tree_maxlevel(tmp_path, root):
    """Tree with maxlevel."""
    UniqueDotExporter(root, maxlevel=2).to_dotfile(tmp_path / "tree_maxlevel.dot")
    assert_gen(tmp_path, REFDATA / "tree_maxlevel")


def test_esc():
    """Test proper escape of quotes."""
    n = Node(r'6"-6\"')
    assert tuple(UniqueDotExporter(n)) == (
        "digraph tree {",
        '    "0x0" [label="6"-6\\""];',
        "}",
    )



import json

from .dictexporter import DictExporter


class JsonExporter:
    """
    Tree to JSON exporter.

    The tree is converted to a dictionary via `dictexporter` and exported to JSON.

    Keyword Arguments:
        dictexporter: Dictionary Exporter used (see :any:`DictExporter`).
        maxlevel (int): Limit export to this number of levels.
        kwargs: All other arguments are passed to
                :any:`json.dump`/:any:`json.dumps`.
                See documentation for reference.

    >>> from anytree import AnyNode
    >>> from anytree.exporter import JsonExporter
    >>> root = AnyNode(a="root")
    >>> s0 = AnyNode(a="sub0", parent=root)
    >>> s0a = AnyNode(a="sub0A", b="foo", parent=s0)
    >>> s0b = AnyNode(a="sub0B", parent=s0)
    >>> s1 = AnyNode(a="sub1", parent=root)

    >>> exporter = JsonExporter(indent=2, sort_keys=True)
    >>> print(exporter.export(root))
    {
      "a": "root",
      "children": [
        {
          "a": "sub0",
          "children": [
            {
              "a": "sub0A",
              "b": "foo"
            },
            {
              "a": "sub0B"
            }
          ]
        },
        {
          "a": "sub1"
        }
      ]
    }

    .. note:: Whenever the json output does not meet your expections, see the :any:`json` documentation.
              For instance, if you have unicode/ascii issues, please try `JsonExporter(..., ensure_ascii=False)`.
    """

    def __init__(self, dictexporter=None, maxlevel=None, **kwargs):
        self.dictexporter = dictexporter
        self.maxlevel = maxlevel
        self.kwargs = kwargs

    def _export(self, node):
        dictexporter = self.dictexporter or DictExporter()
        if self.maxlevel is not None:
            dictexporter.maxlevel = self.maxlevel
        return dictexporter.export(node)

    def export(self, node):
        """Return JSON for tree starting at `node`."""
        data = self._export(node)
        return json.dumps(data, **self.kwargs)

    def write(self, node, filehandle):
        """Write JSON to `filehandle` starting at `node`."""
        data = self._export(node)
        return json.dump(data, filehandle, **self.kwargs)



# -*- coding: utf-8 -*-
import pickle

from anytree import AnyNode, LoopError, Node, NodeMixin, PostOrderIter, PreOrderIter, RenderTree, SymlinkNode, TreeError


def test_pickle(tmp_path):
    """Pickling Compatibilty."""
    root = Node(name="root")
    a = Node(name="a", parent=root)
    b = Node(name="b", parent=a)
    c = SymlinkNode(target=b, parent=a)

    lines = str(RenderTree(root)).splitlines()
    assert lines == [
        "Node('/root')",
        "└── Node('/root/a')",
        "    ├── Node('/root/a/b')",
        "    └── SymlinkNode(Node('/root/a/b'))",
    ]

    filepath = tmp_path / "test.pkl"
    with open(filepath, "wb") as file:
        pickle.dump(root, file)

    with open(filepath, "rb") as file:
        loaded = pickle.load(file)

    assert str(RenderTree(root)).splitlines() == lines



# -*- coding: utf-8 -*-
import anytree


def test_render_str():
    """Render string cast."""
    root = anytree.Node("root")
    s0 = anytree.Node("sub0", parent=root)
    anytree.Node("sub0B", parent=s0)
    anytree.Node("sub0A", parent=s0)
    anytree.Node("sub1", parent=root)

    r = anytree.RenderTree(root)
    assert str(r).splitlines() == [
        "Node('/root')",
        "├── Node('/root/sub0')",
        "│   ├── Node('/root/sub0/sub0B')",
        "│   └── Node('/root/sub0/sub0A')",
        "└── Node('/root/sub1')",
    ]

    r = anytree.RenderTree(root, childiter=lambda nodes: [n for n in nodes if len(n.name) < 5])
    assert str(r).splitlines() == [
        "Node('/root')",
        "├── Node('/root/sub0')",
        "└── Node('/root/sub1')",
    ]


def test_render_repr():
    """Render representation."""
    root = anytree.Node("root")
    anytree.Node("sub", parent=root)
    r = anytree.RenderTree(root)

    assert repr(r) == "RenderTree(Node('/root'), style=ContStyle(), " "childiter=<class 'list'>)"


def test_render():
    """Rendering."""
    root = anytree.Node("root", lines=["c0fe", "c0de"])
    s0 = anytree.Node("sub0", parent=root, lines=["ha", "ba"])
    s0b = anytree.Node("sub0B", parent=s0, lines=["1", "2", "3"])
    s0a = anytree.Node("sub0A", parent=s0, lines=["a", "b"])
    s1 = anytree.Node("sub1", parent=root, lines=["Z"])

    r = anytree.RenderTree(root, style=anytree.DoubleStyle)
    result = [(pre, node) for pre, _, node in r]
    assert result == [
        ("", root),
        ("╠══ ", s0),
        ("║   ╠══ ", s0b),
        ("║   ╚══ ", s0a),
        ("╚══ ", s1),
    ]

    def multi(root):
        for pre, fill, node in anytree.RenderTree(root):
            yield "%s%s" % (pre, node.lines[0]), node
            for line in node.lines[1:]:
                yield "%s%s" % (fill, line), node

    assert list(multi(root)) == [
        ("c0fe", root),
        ("c0de", root),
        ("├── ha", s0),
        ("│   ba", s0),
        ("│   ├── 1", s0b),
        ("│   │   2", s0b),
        ("│   │   3", s0b),
        ("│   └── a", s0a),
        ("│       b", s0a),
        ("└── Z", s1),
    ]


def test_maxlevel():
    root = anytree.Node("root", lines=["c0fe", "c0de"])
    s0 = anytree.Node("sub0", parent=root, lines=["ha", "ba"])
    s0b = anytree.Node("sub0B", parent=s0, lines=["1", "2", "3"])
    s0a = anytree.Node("sub0A", parent=s0, lines=["a", "b"])
    s1 = anytree.Node("sub1", parent=root, lines=["Z"])

    r = anytree.RenderTree(root, maxlevel=2)
    result = [(pre, node) for pre, _, node in r]
    assert result == [
        ("", root),
        ("├── ", s0),
        ("└── ", s1),
    ]


def test_asciistyle():
    style = anytree.AsciiStyle()
    assert style.vertical == "|   "
    assert style.cont == "|-- "
    assert style.end == "+-- "


def test_contstyle():
    style = anytree.ContStyle()
    assert style.vertical == "\u2502   "
    assert style.cont == "\u251c\u2500\u2500 "
    assert style.end == "\u2514\u2500\u2500 "


def test_controundstyle():
    style = anytree.ContRoundStyle()
    assert style.vertical == "\u2502   "
    assert style.cont == "\u251c\u2500\u2500 "
    assert style.end == "\u2570\u2500\u2500 "


def test_doublestyle():
    style = anytree.DoubleStyle()
    assert style.vertical == "\u2551   "
    assert style.cont == "\u2560\u2550\u2550 "
    assert style.end == "\u255a\u2550\u2550 "


def test_by_attr():
    """by attr."""
    root = anytree.Node("root", lines=["root"])
    s0 = anytree.Node("sub0", parent=root, lines=["su", "b0"])
    anytree.Node("sub0B", parent=s0, lines=["sub", "0B"])
    anytree.Node("sub0A", parent=s0)
    anytree.Node("sub1", parent=root, lines=["sub1"])

    assert anytree.RenderTree(root).by_attr().splitlines() == [
        "root",
        "├── sub0",
        "│   ├── sub0B",
        "│   └── sub0A",
        "└── sub1",
    ]
    assert anytree.RenderTree(root).by_attr("lines").splitlines() == [
        "root",
        "├── su",
        "│   b0",
        "│   ├── sub",
        "│   │   0B",
        "│   └── ",
        "└── sub1",
    ]
    assert anytree.RenderTree(root).by_attr(lambda node: ":".join(node.name)).splitlines() == [
        "r:o:o:t",
        "├── s:u:b:0",
        "│   ├── s:u:b:0:B",
        "│   └── s:u:b:0:A",
        "└── s:u:b:1",
    ]


def test_repr():
    """Repr."""

    class ReprNode(anytree.Node):
        def __repr__(self):
            return "{name}\n{name}".format(name=self.name)

    root = ReprNode("root", lines=["root"])
    s0 = ReprNode("sub0", parent=root, lines=["su", "b0"])
    ReprNode("sub0B", parent=s0, lines=["sub", "0B"])
    ReprNode("sub0A", parent=s0)
    ReprNode("sub1", parent=root, lines=["sub1"])

    bystr = str(anytree.RenderTree(root)).splitlines()
    byident = anytree.RenderTree(root).by_attr(lambda node: node).splitlines()
    assert bystr == byident



from anytree import AnyNode, Node, NodeMixin
from anytree.exporter import DictExporter

from .helper import eq_


def test_dict_exporter():
    """Dict Exporter."""
    root = AnyNode(id="root")
    s0 = AnyNode(id="sub0", parent=root)
    s0b = AnyNode(id="sub0B", parent=s0)
    s0a = AnyNode(id="sub0A", parent=s0)
    s1 = AnyNode(id="sub1", parent=root, foo="bar")
    s1a = AnyNode(id="sub1A", parent=s1)
    s1b = AnyNode(id="sub1B", parent=s1)
    s1c = AnyNode(id="sub1C", parent=s1)
    s1ca = AnyNode(id="sub1Ca", parent=s1c)

    exporter = DictExporter()
    eq_(
        exporter.export(root),
        {
            "id": "root",
            "children": [
                {"id": "sub0", "children": [{"id": "sub0B"}, {"id": "sub0A"}]},
                {
                    "id": "sub1",
                    "foo": "bar",
                    "children": [{"id": "sub1A"}, {"id": "sub1B"}, {"id": "sub1C", "children": [{"id": "sub1Ca"}]}],
                },
            ],
        },
    )


def test_dict_exporter_node():
    """Dict Exporter."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root, foo="bar")
    s1a = Node("sub1A", parent=s1)
    s1b = Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    exporter = DictExporter()
    eq_(
        exporter.export(root),
        {
            "name": "root",
            "children": [
                {"name": "sub0", "children": [{"name": "sub0B"}, {"name": "sub0A"}]},
                {
                    "name": "sub1",
                    "foo": "bar",
                    "children": [
                        {"name": "sub1A"},
                        {"name": "sub1B"},
                        {"name": "sub1C", "children": [{"name": "sub1Ca"}]},
                    ],
                },
            ],
        },
    )


def test_dict_exporter_filter():
    """Dict Exporter."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    s0b = Node("sub0B", parent=s0)
    s0a = Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root, foo="bar")
    s1a = Node("sub1A", parent=s1)
    s1b = Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    s1ca = Node("sub1Ca", parent=s1c)

    exporter = DictExporter(attriter=lambda attrs: [(k, v) for k, v in attrs if k == "name"])
    eq_(
        exporter.export(root),
        {
            "name": "root",
            "children": [
                {"name": "sub0", "children": [{"name": "sub0B"}, {"name": "sub0A"}]},
                {
                    "name": "sub1",
                    "children": [
                        {"name": "sub1A"},
                        {"name": "sub1B"},
                        {"name": "sub1C", "children": [{"name": "sub1Ca"}]},
                    ],
                },
            ],
        },
    )


def test_dict_exporter_mixin():
    """Dict Exporter."""

    class MyClass(NodeMixin):
        def __init__(self, foo, parent=None):
            super(MyClass, self).__init__()
            self.foo = foo
            self.parent = parent

    root = MyClass("root")
    s0 = MyClass("s0", parent=root)
    s0b = MyClass("s0b", parent=s0)
    s0a = MyClass("s0a", parent=s0)
    s1 = MyClass("s1", parent=root)
    s1a = MyClass("s1a", parent=s1)
    s1b = MyClass("s1b", parent=s1)
    s1c = MyClass("s1c", parent=s1)
    s1ca = MyClass("s1ca", parent=s1c)

    exporter = DictExporter()
    eq_(
        exporter.export(root),
        {
            "foo": "root",
            "children": [
                {"foo": "s0", "children": [{"foo": "s0b"}, {"foo": "s0a"}]},
                {
                    "foo": "s1",
                    "children": [{"foo": "s1a"}, {"foo": "s1b"}, {"foo": "s1c", "children": [{"foo": "s1ca"}]}],
                },
            ],
        },
    )



# -*- coding: utf-8 -*-
import json

from .dictimporter import DictImporter


class JsonImporter:
    """
    Import Tree from JSON.

    The JSON is read and converted to a dictionary via `dictimporter`.

    Keyword Arguments:
        dictimporter: Dictionary Importer used (see :any:`DictImporter`).
        kwargs: All other arguments are passed to
                :any:`json.load`/:any:`json.loads`.
                See documentation for reference.

    >>> from anytree.importer import JsonImporter
    >>> from anytree import RenderTree
    >>> importer = JsonImporter()
    >>> data = '''
    ... {
    ...   "a": "root",
    ...   "children": [
    ...     {
    ...       "a": "sub0",
    ...       "children": [
    ...         {
    ...           "a": "sub0A",
    ...           "b": "foo"
    ...         },
    ...         {
    ...           "a": "sub0B"
    ...         }
    ...       ]
    ...     },
    ...     {
    ...       "a": "sub1"
    ...     }
    ...   ]
    ... }'''
    >>> root = importer.import_(data)
    >>> print(RenderTree(root))
    AnyNode(a='root')
    ├── AnyNode(a='sub0')
    │   ├── AnyNode(a='sub0A', b='foo')
    │   └── AnyNode(a='sub0B')
    └── AnyNode(a='sub1')
    """

    def __init__(self, dictimporter=None, **kwargs):
        self.dictimporter = dictimporter
        self.kwargs = kwargs

    def __import(self, data):
        dictimporter = self.dictimporter or DictImporter()
        return dictimporter.import_(data)

    def import_(self, data):
        """Read JSON from `data`."""
        return self.__import(json.loads(data, **self.kwargs))

    def read(self, filehandle):
        """Read JSON from `filehandle`."""
        return self.__import(json.load(filehandle, **self.kwargs))



# -*- coding: utf-8 -*-

from anytree import Node, RenderTree

from .helper import eq_


def test_stackoverflow():
    """Example from stackoverflow."""
    udo = Node("Udo")
    marc = Node("Marc", parent=udo)
    Node("Lian", parent=marc)
    dan = Node("Dan", parent=udo)
    Node("Jet", parent=dan)
    Node("Jan", parent=dan)
    joe = Node("Joe", parent=dan)

    eq_(str(udo), "Node('/Udo')")
    eq_(str(joe), "Node('/Udo/Dan/Joe')")

    eq_(
        ["%s%s" % (pre, node.name) for pre, fill, node in RenderTree(udo)],
        [
            "Udo",
            "├── Marc",
            "│   └── Lian",
            "└── Dan",
            "    ├── Jet",
            "    ├── Jan",
            "    └── Joe",
        ],
    )
    eq_(str(dan.children), "(Node('/Udo/Dan/Jet'), Node('/Udo/Dan/Jan'), Node('/Udo/Dan/Joe'))")



import pytest

# https://stackoverflow.com/questions/46962007/how-to-automatically-change-to-pytest-temporary-directory-for-all-doctests


@pytest.fixture(autouse=True)
def _docdir(request):

    # Trigger ONLY for the doctests.
    doctest_plugin = request.config.pluginmanager.getplugin("doctest")
    if isinstance(request.node, doctest_plugin.DoctestItem):

        # Get the fixture dynamically by its name.
        tmpdir = request.getfixturevalue("tmpdir")

        # Chdir only for the duration of the test.
        with tmpdir.as_cwd():
            yield

    else:
        # For normal tests, we have to yield, since this is a yield-fixture.
        yield



"""
Node Classes.

* :any:`AnyNode`: a generic tree node with any number of attributes.
* :any:`Node`: a simple tree node with at least a name attribute and any number of additional attributes.
* :any:`NodeMixin`: extends any python class to a tree node.
* :any:`SymlinkNode`: Tree node which references to another tree node.
* :any:`SymlinkNodeMixin`: extends any Python class to a symbolic link to a tree node.
* :any:`LightNodeMixin`: A :any:`NodeMixin` using slots.
"""

from .anynode import AnyNode  # noqa
from .exceptions import LoopError  # noqa
from .exceptions import TreeError  # noqa
from .lightnodemixin import LightNodeMixin  # noqa
from .node import Node  # noqa
from .nodemixin import NodeMixin  # noqa
from .symlinknode import SymlinkNode  # noqa
from .symlinknodemixin import SymlinkNodeMixin  # noqa



class DictExporter:
    """
    Tree to dictionary exporter.

    Every node is converted to a dictionary with all instance
    attributes as key-value pairs.
    Child nodes are exported to the children attribute.
    A list of dictionaries.

    Keyword Args:
        dictcls: class used as dictionary. :any:`dict` by default.
        attriter: attribute iterator for sorting and/or filtering.
        childiter: child iterator for sorting and/or filtering.
        maxlevel (int): Limit export to this number of levels.

    >>> from pprint import pprint  # just for nice printing
    >>> from anytree import AnyNode
    >>> from anytree.exporter import DictExporter
    >>> root = AnyNode(a="root")
    >>> s0 = AnyNode(a="sub0", parent=root)
    >>> s0a = AnyNode(a="sub0A", b="foo", parent=s0)
    >>> s0b = AnyNode(a="sub0B", parent=s0)
    >>> s1 = AnyNode(a="sub1", parent=root)

    >>> exporter = DictExporter()
    >>> pprint(exporter.export(root))
    {'a': 'root',
     'children': [{'a': 'sub0',
                   'children': [{'a': 'sub0A', 'b': 'foo'}, {'a': 'sub0B'}]},
                  {'a': 'sub1'}]}

    The attribute iterator `attriter` may be used for filtering too.
    For example, just dump attributes named `a`:

    >>> exporter = DictExporter(attriter=lambda attrs: [(k, v) for k, v in attrs if k == "a"])
    >>> pprint(exporter.export(root))
    {'a': 'root',
     'children': [{'a': 'sub0', 'children': [{'a': 'sub0A'}, {'a': 'sub0B'}]},
                  {'a': 'sub1'}]}

    The child iterator `childiter` can be used for sorting and filtering likewise:

    >>> exporter = DictExporter(childiter=lambda children: [child for child in children if "0" in child.a])
    >>> pprint(exporter.export(root))
    {'a': 'root',
     'children': [{'a': 'sub0',
                   'children': [{'a': 'sub0A', 'b': 'foo'}, {'a': 'sub0B'}]}]}
    """

    def __init__(self, dictcls=dict, attriter=None, childiter=list, maxlevel=None):
        self.dictcls = dictcls
        self.attriter = attriter
        self.childiter = childiter
        self.maxlevel = maxlevel

    def export(self, node):
        """Export tree starting at `node`."""
        attriter = self.attriter or (lambda attr_values: attr_values)
        return self.__export(node, self.dictcls, attriter, self.childiter)

    def __export(self, node, dictcls, attriter, childiter, level=1):
        attr_values = attriter(self._iter_attr_values(node))
        data = dictcls(attr_values)
        maxlevel = self.maxlevel
        if maxlevel is None or level < maxlevel:
            children = [
                self.__export(child, dictcls, attriter, childiter, level=level + 1)
                for child in childiter(node.children)
            ]
            if children:
                data["children"] = children
        return data

    @staticmethod
    def _iter_attr_values(node):
        # pylint: disable=C0103
        for k, v in node.__dict__.items():
            if k in ("_NodeMixin__children", "_NodeMixin__parent"):
                continue
            yield k, v



# -*- coding: utf-8 -*-

from .nodemixin import NodeMixin
from .util import _repr


class AnyNode(NodeMixin):
    """
    A generic tree node with any `kwargs`.

    Keyword Args:
        parent: Reference to parent node.
        children: Iterable with child nodes.
        *: Any other given attribute is just stored as object attribute.

    Other than :any:`Node` this class has no default identifier.
    It is up to the user to use other attributes for identification.

    The `parent` attribute refers the parent node:

    >>> from anytree import AnyNode, RenderTree
    >>> root = AnyNode(id="root")
    >>> s0 = AnyNode(id="sub0", parent=root)
    >>> s0b = AnyNode(id="sub0B", parent=s0, foo=4, bar=109)
    >>> s0a = AnyNode(id="sub0A", parent=s0)
    >>> s1 = AnyNode(id="sub1", parent=root)
    >>> s1a = AnyNode(id="sub1A", parent=s1)
    >>> s1b = AnyNode(id="sub1B", parent=s1, bar=8)
    >>> s1c = AnyNode(id="sub1C", parent=s1)
    >>> s1ca = AnyNode(id="sub1Ca", parent=s1c)

    >>> root
    AnyNode(id='root')
    >>> s0
    AnyNode(id='sub0')
    >>> print(RenderTree(root))
    AnyNode(id='root')
    ├── AnyNode(id='sub0')
    │   ├── AnyNode(bar=109, foo=4, id='sub0B')
    │   └── AnyNode(id='sub0A')
    └── AnyNode(id='sub1')
        ├── AnyNode(id='sub1A')
        ├── AnyNode(bar=8, id='sub1B')
        └── AnyNode(id='sub1C')
            └── AnyNode(id='sub1Ca')

    >>> print(RenderTree(root))
    AnyNode(id='root')
    ├── AnyNode(id='sub0')
    │   ├── AnyNode(bar=109, foo=4, id='sub0B')
    │   └── AnyNode(id='sub0A')
    └── AnyNode(id='sub1')
        ├── AnyNode(id='sub1A')
        ├── AnyNode(bar=8, id='sub1B')
        └── AnyNode(id='sub1C')
            └── AnyNode(id='sub1Ca')

    Node attributes can be added, modified and deleted the pythonic way:

    >>> root.new = 'a new attribute'
    >>> s0b
    AnyNode(bar=109, foo=4, id='sub0B')
    >>> s0b.bar = 110  # modified
    >>> s0b
    AnyNode(bar=110, foo=4, id='sub0B')
    >>> del s1b.bar
    >>> print(RenderTree(root))
    AnyNode(id='root', new='a new attribute')
    ├── AnyNode(id='sub0')
    │   ├── AnyNode(bar=110, foo=4, id='sub0B')
    │   └── AnyNode(id='sub0A')
    └── AnyNode(id='sub1')
        ├── AnyNode(id='sub1A')
        ├── AnyNode(id='sub1B')
        └── AnyNode(id='sub1C')
            └── AnyNode(id='sub1Ca')

    The same tree can be constructed by using the `children` attribute:

    >>> root = AnyNode(id="root", children=[
    ...     AnyNode(id="sub0", children=[
    ...         AnyNode(id="sub0B", foo=4, bar=109),
    ...         AnyNode(id="sub0A"),
    ...     ]),
    ...     AnyNode(id="sub1", children=[
    ...         AnyNode(id="sub1A"),
    ...         AnyNode(id="sub1B", bar=8),
    ...         AnyNode(id="sub1C", children=[
    ...             AnyNode(id="sub1Ca"),
    ...         ]),
    ...     ]),
    ... ])
    """

    def __init__(self, parent=None, children=None, **kwargs):

        self.__dict__.update(kwargs)
        self.parent = parent
        if children:
            self.children = children

    def __repr__(self):
        return _repr(self)



# -*- coding: utf-8 -*-
from anytree import Node
from anytree.util import commonancestors, leftsibling, rightsibling

from .helper import eq_


def test_commonancestors():
    """commonancestors."""
    udo = Node("Udo")
    marc = Node("Marc", parent=udo)
    lian = Node("Lian", parent=marc)
    dan = Node("Dan", parent=udo)
    jet = Node("Jet", parent=dan)
    joe = Node("Joe", parent=dan)

    eq_(commonancestors(jet, joe), (udo, dan))
    eq_(commonancestors(jet, marc), (udo,))
    eq_(commonancestors(jet), (udo, dan))
    eq_(commonancestors(), ())
    eq_(commonancestors(jet, lian), (udo,))


def test_leftsibling():
    """leftsibling."""
    dan = Node("Dan")
    jet = Node("Jet", parent=dan)
    jan = Node("Jan", parent=dan)
    joe = Node("Joe", parent=dan)
    eq_(leftsibling(dan), None)
    eq_(leftsibling(jet), None)
    eq_(leftsibling(jan), jet)
    eq_(leftsibling(joe), jan)


def test_rightsibling():
    """rightsibling."""
    dan = Node("Dan")
    jet = Node("Jet", parent=dan)
    jan = Node("Jan", parent=dan)
    joe = Node("Joe", parent=dan)
    eq_(rightsibling(dan), None)
    eq_(rightsibling(jet), jan)
    eq_(rightsibling(jan), joe)
    eq_(rightsibling(joe), None)



from tempfile import NamedTemporaryFile

from anytree.exporter import DictExporter
from anytree.importer import JsonImporter

from .helper import eq_


def test_json_importer():
    """Json Importer."""
    refdata = {
        "id": "root",
        "children": [
            {"id": "sub0", "children": [{"id": "sub0B"}, {"id": "sub0A"}]},
            {
                "id": "sub1",
                "children": [{"id": "sub1A"}, {"id": "sub1B"}, {"id": "sub1C", "children": [{"id": "sub1Ca"}]}],
            },
        ],
    }
    lines = [
        "{",
        '  "children": [',
        "    {",
        '      "children": [',
        "        {",
        '          "id": "sub0B"',
        "        },",
        "        {",
        '          "id": "sub0A"',
        "        }",
        "      ],",
        '      "id": "sub0"',
        "    },",
        "    {",
        '      "children": [',
        "        {",
        '          "id": "sub1A"',
        "        },",
        "        {",
        '          "id": "sub1B"',
        "        },",
        "        {",
        '          "children": [',
        "            {",
        '              "id": "sub1Ca"',
        "            }",
        "          ],",
        '          "id": "sub1C"',
        "        }",
        "      ],",
        '      "id": "sub1"',
        "    }",
        "  ],",
        '  "id": "root"',
        "}",
    ]

    imported = DictExporter().export(JsonImporter().import_("\n".join(lines)))
    eq_(refdata, imported)
    with NamedTemporaryFile(mode="w+") as ref:
        ref.write("\n".join(lines))
        ref.seek(0)
        imported = DictExporter().export(JsonImporter().read(ref))
    eq_(refdata, imported)



# -*- coding: utf-8 -*-
from anytree import AnyNode

from ..config import ASSERTIONS


class DictImporter:
    """
    Import Tree from dictionary.

    Every dictionary is converted to an instance of `nodecls`.
    The dictionaries listed in the children attribute are converted
    likewise and added as children.

    Keyword Args:
        nodecls: class used for nodes.

    >>> from anytree.importer import DictImporter
    >>> from anytree import RenderTree
    >>> importer = DictImporter()
    >>> data = {
    ...     'a': 'root',
    ...     'children': [{'a': 'sub0',
    ...                   'children': [{'a': 'sub0A', 'b': 'foo'}, {'a': 'sub0B'}]},
    ...                  {'a': 'sub1'}]}
    >>> root = importer.import_(data)
    >>> print(RenderTree(root))
    AnyNode(a='root')
    ├── AnyNode(a='sub0')
    │   ├── AnyNode(a='sub0A', b='foo')
    │   └── AnyNode(a='sub0B')
    └── AnyNode(a='sub1')
    """

    def __init__(self, nodecls=AnyNode):
        self.nodecls = nodecls

    def import_(self, data):
        """Import tree from `data`."""
        return self.__import(data)

    def __import(self, data, parent=None):
        if ASSERTIONS:  # pragma: no branch
            assert isinstance(data, dict)
            assert "parent" not in data
        attrs = dict(data)
        children = attrs.pop("children", [])
        node = self.nodecls(parent=parent, **attrs)
        for child in children:
            self.__import(child, parent=node)
        return node



# -*- coding: utf-8 -*-
"""
Tree Rendering.

* :any:`RenderTree` using the following styles:
    * :any:`AsciiStyle`
    * :any:`ContStyle`
    * :any:`ContRoundStyle`
    * :any:`DoubleStyle`
"""

import collections

import six

from .config import ASSERTIONS

Row = collections.namedtuple("Row", ("pre", "fill", "node"))


class AbstractStyle:
    """
    Tree Render Style.

    Args:

        vertical: Sign for vertical line.

        cont: Chars for a continued branch.

        end: Chars for the last branch.
    """

    def __init__(self, vertical, cont, end):
        super(AbstractStyle, self).__init__()
        self.vertical = vertical
        self.cont = cont
        self.end = end
        if ASSERTIONS:  # pragma: no branch
            assert len(cont) == len(vertical) == len(end), "'%s', '%s' and '%s' need to have equal length" % (
                vertical,
                cont,
                end,
            )

    @property
    def empty(self):
        """Empty string as placeholder."""
        return " " * len(self.end)

    def __repr__(self):
        classname = self.__class__.__name__
        return "%s()" % classname


class AsciiStyle(AbstractStyle):
    """
    Ascii style.

    >>> from anytree import Node, RenderTree
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root)
    >>> s0b = Node("sub0B", parent=s0)
    >>> s0a = Node("sub0A", parent=s0)
    >>> s1 = Node("sub1", parent=root)

    >>> print(RenderTree(root, style=AsciiStyle()))
    Node('/root')
    |-- Node('/root/sub0')
    |   |-- Node('/root/sub0/sub0B')
    |   +-- Node('/root/sub0/sub0A')
    +-- Node('/root/sub1')
    """

    def __init__(self):
        super(AsciiStyle, self).__init__("|   ", "|-- ", "+-- ")


class ContStyle(AbstractStyle):
    """
    Continued style, without gaps.

    >>> from anytree import Node, RenderTree
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root)
    >>> s0b = Node("sub0B", parent=s0)
    >>> s0a = Node("sub0A", parent=s0)
    >>> s1 = Node("sub1", parent=root)

    >>> print(RenderTree(root, style=ContStyle()))
    Node('/root')
    ├── Node('/root/sub0')
    │   ├── Node('/root/sub0/sub0B')
    │   └── Node('/root/sub0/sub0A')
    └── Node('/root/sub1')
    """

    def __init__(self):
        super(ContStyle, self).__init__("\u2502   ", "\u251c\u2500\u2500 ", "\u2514\u2500\u2500 ")


class ContRoundStyle(AbstractStyle):
    """
    Continued style, without gaps, round edges.

    >>> from anytree import Node, RenderTree
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root)
    >>> s0b = Node("sub0B", parent=s0)
    >>> s0a = Node("sub0A", parent=s0)
    >>> s1 = Node("sub1", parent=root)

    >>> print(RenderTree(root, style=ContRoundStyle()))
    Node('/root')
    ├── Node('/root/sub0')
    │   ├── Node('/root/sub0/sub0B')
    │   ╰── Node('/root/sub0/sub0A')
    ╰── Node('/root/sub1')
    """

    def __init__(self):
        super(ContRoundStyle, self).__init__("\u2502   ", "\u251c\u2500\u2500 ", "\u2570\u2500\u2500 ")


class DoubleStyle(AbstractStyle):
    """
    Double line style, without gaps.

    >>> from anytree import Node, RenderTree
    >>> root = Node("root")
    >>> s0 = Node("sub0", parent=root)
    >>> s0b = Node("sub0B", parent=s0)
    >>> s0a = Node("sub0A", parent=s0)
    >>> s1 = Node("sub1", parent=root)

    >>> print(RenderTree(root, style=DoubleStyle))
    Node('/root')
    ╠══ Node('/root/sub0')
    ║   ╠══ Node('/root/sub0/sub0B')
    ║   ╚══ Node('/root/sub0/sub0A')
    ╚══ Node('/root/sub1')

    """

    def __init__(self):
        super(DoubleStyle, self).__init__("\u2551   ", "\u2560\u2550\u2550 ", "\u255a\u2550\u2550 ")


@six.python_2_unicode_compatible
class RenderTree:
    """
    Render tree starting at `node`.

    Keyword Args:
        style (AbstractStyle): Render Style.
        childiter: Child iterator.
        maxlevel: Limit rendering to this depth.

    :any:`RenderTree` is an iterator, returning a tuple with 3 items:

    `pre`
        tree prefix.

    `fill`
        filling for multiline entries.

    `node`
        :any:`NodeMixin` object.

    It is up to the user to assemble these parts to a whole.

    >>> from anytree import Node, RenderTree
    >>> root = Node("root", lines=["c0fe", "c0de"])
    >>> s0 = Node("sub0", parent=root, lines=["ha", "ba"])
    >>> s0b = Node("sub0B", parent=s0, lines=["1", "2", "3"])
    >>> s0a = Node("sub0A", parent=s0, lines=["a", "b"])
    >>> s1 = Node("sub1", parent=root, lines=["Z"])

    Simple one line:

    >>> for pre, _, node in RenderTree(root):
    ...     print("%s%s" % (pre, node.name))
    root
    ├── sub0
    │   ├── sub0B
    │   └── sub0A
    └── sub1

    Multiline:

    >>> for pre, fill, node in RenderTree(root):
    ...     print("%s%s" % (pre, node.lines[0]))
    ...     for line in node.lines[1:]:
    ...         print("%s%s" % (fill, line))
    c0fe
    c0de
    ├── ha
    │   ba
    │   ├── 1
    │   │   2
    │   │   3
    │   └── a
    │       b
    └── Z

    `maxlevel` limits the depth of the tree:

    >>> print(RenderTree(root, maxlevel=2))
    Node('/root', lines=['c0fe', 'c0de'])
    ├── Node('/root/sub0', lines=['ha', 'ba'])
    └── Node('/root/sub1', lines=['Z'])

    The `childiter` is responsible for iterating over child nodes at the
    same level. An reversed order can be achived by using `reversed`.

    >>> for row in RenderTree(root, childiter=reversed):
    ...     print("%s%s" % (row.pre, row.node.name))
    root
    ├── sub1
    └── sub0
        ├── sub0A
        └── sub0B

    Or writing your own sort function:

    >>> def mysort(items):
    ...     return sorted(items, key=lambda item: item.name)
    >>> for row in RenderTree(root, childiter=mysort):
    ...     print("%s%s" % (row.pre, row.node.name))
    root
    ├── sub0
    │   ├── sub0A
    │   └── sub0B
    └── sub1

    :any:`by_attr` simplifies attribute rendering and supports multiline:

    >>> print(RenderTree(root).by_attr())
    root
    ├── sub0
    │   ├── sub0B
    │   └── sub0A
    └── sub1
    >>> print(RenderTree(root).by_attr("lines"))
    c0fe
    c0de
    ├── ha
    │   ba
    │   ├── 1
    │   │   2
    │   │   3
    │   └── a
    │       b
    └── Z

    And can be a function:

    >>> print(RenderTree(root).by_attr(lambda n: " ".join(n.lines)))
    c0fe c0de
    ├── ha ba
    │   ├── 1 2 3
    │   └── a b
    └── Z
    """

    def __init__(self, node, style=ContStyle(), childiter=list, maxlevel=None):
        if not isinstance(style, AbstractStyle):
            style = style()
        self.node = node
        self.style = style
        self.childiter = childiter
        self.maxlevel = maxlevel

    def __iter__(self):
        return self.__next(self.node, tuple())

    def __next(self, node, continues, level=0):
        yield RenderTree.__item(node, continues, self.style)
        level += 1
        if self.maxlevel is None or level < self.maxlevel:
            children = node.children
            if children:
                children = self.childiter(children)
                for child, is_last in _is_last(children):
                    for grandchild in self.__next(child, continues + (not is_last,), level=level):
                        yield grandchild

    @staticmethod
    def __item(node, continues, style):
        if not continues:
            return Row("", "", node)
        items = [style.vertical if cont else style.empty for cont in continues]
        indent = "".join(items[:-1])
        branch = style.cont if continues[-1] else style.end
        pre = indent + branch
        fill = "".join(items)
        return Row(pre, fill, node)

    def __str__(self):
        def get():
            for row in self:
                lines = repr(row.node).splitlines() or [""]
                yield "%s%s" % (row.pre, lines[0])
                for line in lines[1:]:
                    yield "%s%s" % (row.fill, line)

        return "\n".join(get())

    def __repr__(self):
        classname = self.__class__.__name__
        args = [repr(self.node), "style=%s" % repr(self.style), "childiter=%s" % repr(self.childiter)]
        return "%s(%s)" % (classname, ", ".join(args))

    def by_attr(self, attrname="name"):
        """
        Return rendered tree with node attribute `attrname`.

        >>> from anytree import AnyNode, RenderTree
        >>> root = AnyNode(id="root")
        >>> s0 = AnyNode(id="sub0", parent=root)
        >>> s0b = AnyNode(id="sub0B", parent=s0, foo=4, bar=109)
        >>> s0a = AnyNode(id="sub0A", parent=s0)
        >>> s1 = AnyNode(id="sub1", parent=root)
        >>> s1a = AnyNode(id="sub1A", parent=s1)
        >>> s1b = AnyNode(id="sub1B", parent=s1, bar=8)
        >>> s1c = AnyNode(id="sub1C", parent=s1)
        >>> s1ca = AnyNode(id="sub1Ca", parent=s1c)
        >>> print(RenderTree(root).by_attr('id'))
        root
        ├── sub0
        │   ├── sub0B
        │   └── sub0A
        └── sub1
            ├── sub1A
            ├── sub1B
            └── sub1C
                └── sub1Ca

        """

        def get():
            if callable(attrname):
                for row in self:
                    attr = attrname(row.node)
                    yield from _format_row_any(row, attr)
            else:
                for row in self:
                    attr = getattr(row.node, attrname, "")
                    yield from _format_row_any(row, attr)

        return "\n".join(get())


def _format_row_any(row, attr):
    if isinstance(attr, (list, tuple)):
        lines = attr or [""]
    else:
        lines = str(attr).splitlines() or [""]
    yield "%s%s" % (row.pre, lines[0])
    for line in lines[1:]:
        yield "%s%s" % (row.fill, line)


def _is_last(iterable):
    iter_ = iter(iterable)
    try:
        nextitem = next(iter_)
    except StopIteration:
        pass
    else:
        item = nextitem
        while True:
            try:
                nextitem = next(iter_)
                yield item, False
            except StopIteration:
                yield nextitem, True
                break
            item = nextitem



# -*- coding: utf-8 -*-
"""Test custom node separator."""

import six

import anytree as at

from .helper import assert_raises, eq_


class MyNode(at.Node):

    separator = "|"


def test_render():
    """Render string cast."""
    root = MyNode("root")
    s0 = MyNode("sub0", parent=root)
    MyNode("sub0B", parent=s0)
    MyNode("sub0A", parent=s0)
    MyNode("sub1", parent=root)
    r = at.RenderTree(root)

    assert str(r).splitlines() == [
        "MyNode('|root')",
        "├── MyNode('|root|sub0')",
        "│   ├── MyNode('|root|sub0|sub0B')",
        "│   └── MyNode('|root|sub0|sub0A')",
        "└── MyNode('|root|sub1')",
    ]


def test_get():
    """Get."""
    top = MyNode("top", parent=None)
    sub0 = MyNode("sub0", parent=top)
    sub0sub0 = MyNode("sub0sub0", parent=sub0)
    sub0sub1 = MyNode("sub0sub1", parent=sub0)
    sub1 = MyNode("sub1", parent=top)
    r = at.Resolver("name")
    eq_(r.get(top, "sub0|sub0sub0"), sub0sub0)
    eq_(r.get(sub1, ".."), top)
    eq_(r.get(sub1, "..|sub0|sub0sub1"), sub0sub1)
    eq_(r.get(sub1, "."), sub1)
    eq_(r.get(sub1, ""), sub1)
    with assert_raises(at.ChildResolverError, "MyNode('|top') has no child sub2. Children are: 'sub0', 'sub1'."):
        r.get(top, "sub2")
    eq_(r.get(sub0sub0, "|top"), top)
    eq_(r.get(sub0sub0, "|top|sub0"), sub0)
    with assert_raises(at.ResolverError, "root node missing. root is '|top'."):
        r.get(sub0sub0, "|")
    with assert_raises(at.ResolverError, "unknown root node '|bar'. root is '|top'."):
        r.get(sub0sub0, "|bar")


def test_glob():
    """Wildcard."""
    top = MyNode("top", parent=None)
    sub0 = MyNode("sub0", parent=top)
    sub0sub0 = MyNode("sub0", parent=sub0)
    sub0sub1 = MyNode("sub1", parent=sub0)
    sub0sub1sub0 = MyNode("sub0", parent=sub0sub1)
    MyNode("sub1", parent=sub0sub1)
    sub1 = MyNode("sub1", parent=top)
    sub1sub0 = MyNode("sub0", parent=sub1)
    r = at.Resolver()
    eq_(r.glob(top, "*|*|sub0"), [sub0sub1sub0])

    eq_(r.glob(top, "sub0|sub?"), [sub0sub0, sub0sub1])
    eq_(r.glob(sub1, "..|.|*"), [sub0, sub1])
    eq_(r.glob(top, "*|*"), [sub0sub0, sub0sub1, sub1sub0])
    eq_(r.glob(top, "*|sub0"), [sub0sub0, sub1sub0])
    with assert_raises(at.ChildResolverError, "MyNode('|top|sub1') has no child sub1. Children are: 'sub0'."):
        r.glob(top, "sub1|sub1")



import filecmp
import pathlib
import shutil

LEARN = False


def assert_gen(genpath, refpath):
    """Compare Generated Files Versus Reference."""
    genpath.mkdir(parents=True, exist_ok=True)
    refpath.mkdir(parents=True, exist_ok=True)
    if LEARN:  # pragma: no cover
        shutil.rmtree(refpath, ignore_errors=True)
        shutil.copytree(genpath, refpath)

    gens = [path for path in sorted(genpath.glob("**/*")) if path.is_file() and not "__pycache__" in path.parts]
    refs = [path for path in sorted(refpath.glob("**/*")) if path.is_file() and not "__pycache__" in path.parts]

    genfiles = [path.relative_to(genpath) for path in gens]
    reffiles = [path.relative_to(refpath) for path in refs]
    assert reffiles == genfiles, f"{reffiles} != {genfiles}"

    for gen, ref in zip(gens, refs):
        reftext = ref.read_text(encoding="utf-8")
        gentext = gen.read_text(encoding="utf-8")
        assert reftext == gentext, f"{reftext} != {gentext}"



# -*- coding: utf-8 -*-
from .symlinknodemixin import SymlinkNodeMixin
from .util import _repr


class SymlinkNode(SymlinkNodeMixin):
    """
    Tree node which references to another tree node.

    Args:
        target: Symbolic Link Target. Another tree node, which is refered to.

    Keyword Args:
        parent: Reference to parent node.
        children: Iterable with child nodes.
        *: Any other given attribute is just stored as attribute **in** `target`.

    The :any:`SymlinkNode` has its own parent and its own child nodes.
    All other attribute accesses are just forwarded to the target node.

    >>> from anytree import SymlinkNode, Node, RenderTree
    >>> root = Node("root")
    >>> s1 = Node("sub1", parent=root, bar=17)
    >>> l = SymlinkNode(s1, parent=root, baz=18)
    >>> l0 = Node("l0", parent=l)
    >>> print(RenderTree(root))
    Node('/root')
    ├── Node('/root/sub1', bar=17, baz=18)
    └── SymlinkNode(Node('/root/sub1', bar=17, baz=18))
        └── Node('/root/sub1/l0')

    Any modifications on the target node are also available on the linked node and vice-versa:

    >>> s1.foo = 4
    >>> s1.foo
    4
    >>> l.foo
    4
    >>> l.foo = 9
    >>> s1.foo
    9
    >>> l.foo
    9
    """

    def __init__(self, target, parent=None, children=None, **kwargs):
        self.target = target
        self.target.__dict__.update(kwargs)
        self.parent = parent
        if children:
            self.children = children

    def __repr__(self):
        return _repr(self, [repr(self.target)], nameblacklist=("target",))



import filecmp
import os
from tempfile import NamedTemporaryFile

from anytree import AnyNode
from anytree.exporter import JsonExporter

from .helper import eq_


def test_json_exporter():
    """Json Exporter."""
    root = AnyNode(id="root")
    s0 = AnyNode(id="sub0", parent=root)
    AnyNode(id="sub0B", parent=s0)
    AnyNode(id="sub0A", parent=s0)
    s1 = AnyNode(id="sub1", parent=root)
    AnyNode(id="sub1A", parent=s1)
    AnyNode(id="sub1B", parent=s1)
    s1c = AnyNode(id="sub1C", parent=s1)
    AnyNode(id="sub1Ca", parent=s1c)

    exporter = JsonExporter(indent=2, sort_keys=True)
    exported = exporter.export(root).splitlines()
    exported = [e.rstrip() for e in exported]  # just a fix for a strange py2x behavior.
    lines = [
        "{",
        '  "children": [',
        "    {",
        '      "children": [',
        "        {",
        '          "id": "sub0B"',
        "        },",
        "        {",
        '          "id": "sub0A"',
        "        }",
        "      ],",
        '      "id": "sub0"',
        "    },",
        "    {",
        '      "children": [',
        "        {",
        '          "id": "sub1A"',
        "        },",
        "        {",
        '          "id": "sub1B"',
        "        },",
        "        {",
        '          "children": [',
        "            {",
        '              "id": "sub1Ca"',
        "            }",
        "          ],",
        '          "id": "sub1C"',
        "        }",
        "      ],",
        '      "id": "sub1"',
        "    }",
        "  ],",
        '  "id": "root"',
        "}",
    ]
    eq_(exported, lines)

    exporter = JsonExporter(indent=2, sort_keys=True, maxlevel=2)
    exported = exporter.export(root).splitlines()
    exported = [e.rstrip() for e in exported]  # just a fix for a strange py2x behavior.
    limitedlines = [
        "{",
        '  "children": [',
        "    {",
        '      "id": "sub0"',
        "    },",
        "    {",
        '      "id": "sub1"',
        "    }",
        "  ],",
        '  "id": "root"',
        "}",
    ]

    eq_(exported, limitedlines)

    try:
        with NamedTemporaryFile(mode="w+", delete=False) as ref:
            with NamedTemporaryFile(mode="w+", delete=False) as gen:
                ref.write("\n".join(lines))
                exporter.write(root, gen)
        # on Windows, you must close the files before comparison
        filecmp.cmp(ref.name, gen.name)
    finally:
        os.remove(ref.name)
        os.remove(gen.name)



from filecmp import cmp
from os import makedirs
from os.path import dirname, exists, join
from shutil import rmtree

from anytree import Node
from anytree.dotexport import RenderTreeGraph

from .helper import with_setup

TESTPATH = dirname(__file__)
GENPATH = join(TESTPATH, "dotexport")
REFPATH = join(TESTPATH, "refdata")


def setup():
    if not exists(GENPATH):
        makedirs(GENPATH)


def teardown():
    if exists(GENPATH):
        rmtree(GENPATH)


@with_setup(setup, teardown)
def test_tree1():
    """Tree1."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    Node("sub0B", parent=s0)
    Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    Node("sub1A", parent=s1)
    Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    Node(99, parent=s1c)

    RenderTreeGraph(root).to_dotfile(join(GENPATH, "tree1.dot"))
    assert cmp(join(GENPATH, "tree1.dot"), join(REFPATH, "tree1.dot"))


@with_setup(setup, teardown)
def test_tree2():
    """Tree2."""
    root = Node("root")
    s0 = Node("sub0", parent=root, edge=2)
    Node("sub0B", parent=s0, foo=4, edge=109)
    Node("sub0A", parent=s0, edge="")
    s1 = Node("sub1", parent=root, edge="")
    Node("sub1A", parent=s1, edge=7)
    Node("sub1B", parent=s1, edge=8)
    s1c = Node("sub1C", parent=s1, edge=22)
    Node("sub1Ca", parent=s1c, edge=42)

    def nodenamefunc(node):
        return "%s:%s" % (node.name, node.depth)

    def edgeattrfunc(node, child):
        return 'label="%s:%s"' % (node.name, child.name)

    r = RenderTreeGraph(
        root,
        options=["rankdir=LR;"],
        nodenamefunc=nodenamefunc,
        nodeattrfunc=lambda node: "shape=box",
        edgeattrfunc=edgeattrfunc,
    )

    r.to_dotfile(join(GENPATH, "tree2.dot"))
    assert cmp(join(GENPATH, "tree2.dot"), join(REFPATH, "tree2.dot"))


@with_setup(setup, teardown)
def test_tree_png():
    """Tree to png."""
    root = Node("root")
    s0 = Node("sub0", parent=root)
    Node("sub0B", parent=s0)
    Node("sub0A", parent=s0)
    s1 = Node("sub1", parent=root)
    Node("sub1A", parent=s1)
    Node("sub1B", parent=s1)
    s1c = Node("sub1C", parent=s1)
    Node("sub1Ca", parent=s1c)

    RenderTreeGraph(root).to_picture(join(GENPATH, "tree1.png"))



from anytree import Node, Walker, WalkError

from .helper import assert_raises, eq_


def test_walker():
    """walk test."""
    f = Node("f")
    b = Node("b", parent=f)
    a = Node("a", parent=b)
    d = Node("d", parent=b)
    c = Node("c", parent=d)
    e = Node("e", parent=d)
    g = Node("g", parent=f)
    i = Node("i", parent=g)
    h = Node("h", parent=i)
    w = Walker()
    eq_(w.walk(f, f), ((), f, ()))
    eq_(w.walk(f, b), ((), f, (b,)))
    eq_(w.walk(b, f), ((b,), f, ()))
    eq_(w.walk(a, f), ((a, b), f, ()))
    eq_(w.walk(h, e), ((h, i, g), f, (b, d, e)))
    eq_(w.walk(d, e), ((), d, (e,)))

    with assert_raises(WalkError, "Node('/a') and Node('/b') are not part of the same tree."):
        w.walk(Node("a"), Node("b"))



from enum import IntEnum

from anytree import AsciiStyle, CountError, Node, PreOrderIter, RenderTree, find, find_by_attr, findall, findall_by_attr

from .helper import assert_raises, eq_


def test_findall():
    f = Node("f")
    b = Node("b", parent=f)
    a = Node("a", parent=b)
    d = Node("d", parent=b)
    c = Node("c", parent=d)
    e = Node("e", parent=d)

    eq_(findall(f, filter_=lambda node: node.name in ("a", "b")), (b, a))
    eq_(findall(f, filter_=lambda node: d in node.path), (d, c, e))
    with assert_raises(
        CountError,
        ("Expecting at least 4 elements, but found 3. " "(Node('/f/b/d'), Node('/f/b/d/c'), Node('/f/b/d/e'))"),
    ):
        findall(f, filter_=lambda node: d in node.path, mincount=4)
    with assert_raises(
        CountError,
        ("Expecting 2 elements at maximum, but found 3. " "(Node('/f/b/d'), Node('/f/b/d/c'), Node('/f/b/d/e'))"),
    ):
        findall(f, filter_=lambda node: d in node.path, maxcount=2)


def test_findall_by_attr():
    f = Node("f")
    b = Node("b", parent=f)
    Node("a", parent=b)
    d = Node("d", parent=b)
    Node("c", parent=d)
    Node("e", parent=d)

    eq_(findall_by_attr(f, "d"), (d,))
    with assert_raises(CountError, ("Expecting at least 1 elements, but found 0.")):
        findall_by_attr(f, "z", mincount=1)


def test_find():
    f = Node("f")
    b = Node("b", parent=f)
    Node("a", parent=b)
    d = Node("d", parent=b)
    Node("c", parent=d)
    Node("e", parent=d)
    g = Node("g", parent=f)
    i = Node("i", parent=g)
    Node("h", parent=i)

    eq_(find(f, lambda n: n.name == "d"), d)
    eq_(find(f, lambda n: n.name == "z"), None)
    with assert_raises(
        CountError,
        (
            "Expecting 1 elements at maximum, but found 5. "
            "(Node('/f/b'), Node('/f/b/a'), Node('/f/b/d'), Node('/f/b/d/c'), Node('/f/b/d/e'))"
        ),
    ):
        find(f, lambda n: b in n.path)


def test_find_by_attr():
    f = Node("f")
    b = Node("b", parent=f)
    Node("a", parent=b)
    d = Node("d", parent=b)
    c = Node("c", parent=d, foo=4)
    Node("e", parent=d)
    g = Node("g", parent=f)
    i = Node("i", parent=g)
    Node("h", parent=i)

    eq_(find_by_attr(f, "d"), d)
    eq_(find_by_attr(f, name="foo", value=4), c)
    eq_(find_by_attr(f, name="foo", value=8), None)


def test_enum():
    class Animals(IntEnum):
        Mammal = 1
        Cat = 2
        Dog = 3

    root = Node("ANIMAL")
    mammal = Node(Animals.Mammal, parent=root)
    cat = Node(Animals.Cat, parent=mammal)
    dog = Node(Animals.Dog, parent=mammal)

    eq_(findall(root), (root, mammal, cat, dog))
    eq_(findall_by_attr(root, Animals.Cat), (cat,))
